@@ -17,13 +17,14 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *)
 
+(* TODO: organize this ... *)
+
+include Ocsigen_lib_obrowser
+
 exception Input_is_too_large
 exception Ocsigen_Bad_Request
 exception Ocsigen_Request_too_long
 
-let id x = x
-let (>>=) = Lwt.bind
-
 let comp f g x = f (g x)
 
 let rec list_remove_first_if_any a = function
@@ -58,7 +59,7 @@ let rec list_remove_all_q a = function
 
 let rec list_remove_all_assoc a = function
   | [] -> []
-  | (b,_)::l when a = b -> list_remove_all_assoc a l
+  | (b, _)::l when a = b -> list_remove_all_assoc a l
   | b::l -> b::(list_remove_all_assoc a l)
 
 let rec list_remove_all_assoc_q a = function
@@ -74,9 +75,9 @@ let rec list_last = function
   | _::l -> list_last l
 
 let rec list_assoc_remove a = function
-    [] -> raise Not_found
-  | (b,c)::l when a = b -> c,l
-  | b::l -> let v,ll = list_assoc_remove a l in v,b::ll
+  | [] -> raise Not_found
+  | (b, c)::l when a = b -> c, l
+  | b::l -> let v, ll = list_assoc_remove a l in (v, b::ll)
 
 let rec list_is_prefix l1 l2 =
   match (l1, l2) with
@@ -95,11 +96,12 @@ let rec list_is_prefix_skip_end_slash l1 l2 =
 (** various functions for URLs *)
 
 let remove_dotdot =
-  (* removes "../" and "//" *)
+  (* removes "../" *) 
   let rec aux = function
     | [] -> []
     | [""] as l -> l
-    | ""::l -> aux l
+(*    | ""::l -> aux l *) (* we do not remove "//" any more, 
+                             because of optional suffixes in Eliom *)
     | ".."::l -> aux l
     | a::l -> a::(aux l)
   in function
@@ -188,15 +190,6 @@ let rec string_first_diff s1 s2 n last =
 
 (*****************************************************************************)
 
-let add_to_string s1 sep = function
-  | "" -> s1
-  | s2 -> s1^sep^s2
-
-let concat_strings s1 sep s2 = match s1,s2 with
-| _,"" -> s1
-| "",_ -> s2
-| _ -> s1^sep^s2
-
 (* Cut a string to the next separator *)
 let basic_sep char s =
   try
@@ -275,10 +268,6 @@ let (string_of_exn, register_exn_printer) =
 
 (* Unix.inet_addr is abstract and nothing to convert it :-( *)
 
-type ip_address =
-  | IPv4 of int32
-  | IPv6 of int64 * int64
-
 exception Invalid_ip_address of string
 
 
@@ -418,6 +407,14 @@ let match_ip (base, mask) ip =
         end
     | _ -> false
 
+let inet6_addr_loopback = 
+  fst (parse_ip (Unix.string_of_inet_addr Unix.inet6_addr_loopback))
+
+let network_of_ip ip mask4 (mask61, mask62) = match ip with
+  | IPv4 a -> IPv4 (Int32.logand a mask4)
+  | IPv6 (a, b) -> IPv6 (Int64.logand a mask61, Int64.logand b mask62)
+
+
 (* *)
 let fst3 (a, _, _) = a
 let snd3 (_, a, _) = a
@@ -480,34 +477,39 @@ module MyUrl = struct
 
   let of_hex1 c =
     match c with
-        ('0'..'9') -> Char.code c - Char.code '0'
+      | ('0'..'9') -> Char.code c - Char.code '0'
       | ('A'..'F') -> Char.code c - Char.code 'A' + 10
       | ('a'..'f') -> Char.code c - Char.code 'a' + 10
       | _ ->
         raise Not_found ;;
 
   let url_encoding_re =
-    Netstring_pcre.regexp "[^A-Za-z0-9_.!*-~]";;
+    Netstring_pcre.regexp "[^A-Za-z0-9~_.!*\\-]";;
 
   let encode ?(plus = true) s =
     Netstring_pcre.global_substitute
       url_encoding_re
       (fun r _ ->
          match Netstring_pcre.matched_string r s with
-             " " when plus -> "+"
+           | " " when plus -> "+"
            | x ->
                let k = Char.code(x.[0]) in
                "%" ^ to_hex2 k
       )
-      s ;;
+      s
+
 end
 
+let encode = MyUrl.encode
 
-let string_of_url_path ~encode l = 
+let mk_url_encoded_parameters = Netencoding.Url.mk_url_encoded_parameters
+
+let string_of_url_path ~encode l =
   if encode
   then
     fixup_url_string (String.concat "/"
-                        (List.map (*Netencoding.Url.encode*) MyUrl.encode l))
+                        (List.map (*Netencoding.Url.encode*) 
+                           MyUrl.encode l))
   else String.concat "/" l (* BYXXX : check illicit characters *)
 
 let parse_url =
@@ -593,20 +595,6 @@ let parse_url =
     (https, host, port, url, url2, path, params, get_params)
 
 
-(************************************************************************)
-(* absolute urls *)
-
-let make_absolute_url ~https ~host ~port uri =
-  (if https
-   then "https://"
-   else "http://"
-  )^
-    host^
-    (if (port = 80 && not https) || (https && port = 443)
-     then ""
-     else ":"^string_of_int port)^
-    uri
-
 
 (************************************************************************)
 
@@ -649,3 +637,79 @@ type yesnomaybe = Yes | No | Maybe
 
 module StringSet = Set.Make(String)
 
+
+
+(* circular lists *)
+module Clist = 
+(struct
+  
+  type 'a node =
+      { content : 'a option;
+        mutable prev : 'a node; 
+        mutable next : 'a node }
+
+  type 'a t = 'a node
+        
+  let make' c =
+    let rec x = { content = c; prev = x; next = x } in
+    x
+      
+  let make c = make' (Some c)
+
+  let create () = make' None
+
+  let insert p x =
+    let n = p.next in
+    p.next <- x;
+    x.prev <- p;
+    x.next <- n;
+    n.prev <- x
+      
+  let remove x =
+    let p = x.prev in
+    let n = x.next in
+    p.next <- n;
+    n.prev <- p;
+    x.next <- x;
+    x.prev <- x
+      
+  let in_list x = x.next != x
+    
+  let is_empty set = set.next == set
+
+  let value c = 
+    match c.content with 
+      | None -> failwith "Clist.value"
+      | Some c -> c
+
+  let rec iter f (node : 'a t) = 
+    match node.next.content with
+      | Some c -> 
+          f c;
+          iter f node.next
+      | None -> ()
+
+  let rec fold_left f a (node : 'a t) = 
+    match node.next.content with
+      | Some c ->  fold_left f (f a c) node.next
+      | None -> a
+
+end : sig
+  type 'a t
+  type 'a node
+  val make : 'a -> 'a node
+  val create : unit -> 'a t
+  val insert : 'a t -> 'a node -> unit
+  val remove : 'a node -> unit
+  val value : 'a node -> 'a
+  val in_list : 'a node -> bool
+  val is_empty : 'a t -> bool
+  val iter : ('a -> unit) -> 'a t -> unit
+  val fold_left : ('a -> 'b -> 'a) -> 'a -> 'b t -> 'a
+end)
+
+module Int_Table = Map.Make(struct
+  type t = int
+  let compare = compare
+end)
+
