@@ -101,16 +101,26 @@ module Xhtmlreg_ = struct
 
   type options = unit
 
-  let send ?options ?(cookies=[]) ?charset ?code ~sp content =
+  let send ?options ?(cookies=[]) ?charset ?code
+      ?content_type ?headers ~sp content =
     Ocamlduce_content.result_of_content content >>= fun r ->
     Lwt.return
-        (Eliom_services.EliomResult
-           {r with
-            res_cookies=
-            Eliom_services.cookie_table_of_eliom_cookies ~sp cookies;
-            res_code= code_of_code_option code;
-            res_charset= Some "utf-8" (* For Eliom_duce, we impose utf-8 *);
-          })
+      {r with
+         res_cookies=
+          Eliom_services.cookie_table_of_eliom_cookies ~sp cookies;
+         res_code= code_of_code_option code;
+         res_charset= Some "utf-8" (* For Eliom_duce, we impose utf-8 *);
+         res_content_type= (match content_type with
+                              | None -> r.res_content_type
+                              | _ -> content_type
+                           );
+         res_headers= (match headers with
+                         | None -> r.res_headers
+                         | Some headers -> 
+                             Http_headers.with_defaults 
+                               headers r.res_headers
+                      );
+      }
 
 end
 
@@ -123,8 +133,8 @@ module Xhtmlforms_ = struct
   type a_content_elt_list = {{ [ a_content* ] }}
   type a_elt = a
   type a_elt_list = {{ [ a* ] }}
-  type div_content_elt = flows
-  type div_content_elt_list = {{ [ flows* ] }}
+  type div_content_elt = div_content
+  type div_content_elt_list = flows
   type uri = string
   type link_elt = link
   type script_elt = script
@@ -255,11 +265,11 @@ module Xhtmlforms_ = struct
     in
     {{ <button ({type=button_type} ++ a ++ a2 ++ a3)> c }}
 
-  let make_textarea ?(a={{ {} }}) ~name ?(value={{ [] }}) ~rows ~cols () =
+  let make_textarea ?(a={{ {} }}) ~name ?(value="") ~rows ~cols () =
     {{ <textarea ({ name=(str name)
                     rows={: string_of_int rows :}
                     cols={: string_of_int cols :}
-                  } ++ a)> value }}
+                  } ++ a)> (str value) }}
 
   let make_select ?(a={{ {} }}) ~multiple ~name elt elts =
     let a2 = if multiple then {{ { multiple="multiple" } }} else {{ {} }} in
@@ -300,91 +310,16 @@ end
 (****************************************************************************)
 
 
-
-
-module SubXhtml =
-  functor(T : sig
-            type content
-            val print : (string -> unit ) -> content -> unit
-          end) ->
-  (struct
-
-    module Cont_content =
-      struct
-        type t = T.content
-
-        let get_etag_aux x =
-          Some (Digest.to_hex (Digest.string x))
-
-        let print x =
-          let b = Buffer.create 256 in
-          T.print (Buffer.add_string b) x;
-          Buffer.contents b
-
-        let get_etag c =
-          let x = print c in
-          get_etag_aux x
-
-        let result_of_content c =
-          let x = print c in
-          let md5 = get_etag_aux x in
-          let default_result = default_result () in
-          Lwt.return
-            {default_result with
-             res_content_length = Some (Int64.of_int (String.length x));
-             res_content_type = Some "text/html";
-             res_etag = md5;
-             res_headers= Http_headers.dyn_headers;
-             res_stream =
-                (Ocsigen_stream.make
-                   (fun () -> Ocsigen_stream.cont x
-                      (fun () -> Ocsigen_stream.empty None)),
-                 None)
-           }
-
-      end
-
-
-    module Contreg_ = struct
-      open XHTML.M
-      open Xhtmltypes
-
-      type page = T.content
-
-      type options = unit
-
-      let send ?options ?(cookies=[]) ?charset ?code ~sp content =
-        Cont_content.result_of_content content >>= fun r ->
-        Lwt.return
-            (Eliom_services.EliomResult
-               {r with
-                res_cookies=
-                Eliom_services.cookie_table_of_eliom_cookies ~sp cookies;
-                res_code= code_of_code_option code;
-                res_charset= Some "utf-8" (* For Eliom_duce, we impose utf-8 *);
-               })
-
-    end
-
-    module Contreg = Eliom_mkreg.MakeRegister(Contreg_)
-
-    include Xhtmlforms
-    include Contreg
-
-
-   end : sig
-
-     include Eliom_mkreg.ELIOMREGSIG with type page = T.content
-     include Eliom_mkforms.ELIOMFORMSIG with
-      type form_content_elt = form_content
+module type XhtmlFormsSig = Eliom_mkforms.ELIOMFORMSIG with
+  type form_content_elt = form_content
   and type form_content_elt_list = {{ [ form_content* ] }}
   and type form_elt = form
   and type a_content_elt = a_content
   and type a_content_elt_list = {{ [ a_content* ] }}
   and type a_elt = a
   and type a_elt_list = {{ [ a* ] }}
-  and type div_content_elt = flows
-  and type div_content_elt_list = {{ [ flows* ] }}
+  and type div_content_elt = div_content
+  and type div_content_elt_list = flows
   and type uri = string
   and type link_elt = link
   and type script_elt = script
@@ -422,7 +357,96 @@ module SubXhtml =
   and type input_type_t = input_type_values
   and type button_type_t = button_type_values
 
-   end)
+
+module type XhtmlSig =
+sig
+  include Eliom_mkreg.ELIOMREGSIG with type options = unit
+  include XhtmlFormsSig
+end
+
+module SubXhtml =
+  functor(T : sig
+            type content
+            val print : (string -> unit ) -> content -> unit
+          end) ->
+  (struct
+
+    module Cont_content =
+      struct
+        type t = T.content
+
+        let get_etag_aux x =
+          Some (Digest.to_hex (Digest.string x))
+
+        let print x =
+          let b = Buffer.create 256 in
+          T.print (Buffer.add_string b) x;
+          Buffer.contents b
+
+        let get_etag c =
+          let x = print c in
+          get_etag_aux x
+
+        let result_of_content c =
+          let x = print c in
+          let md5 = get_etag_aux x in
+          let default_result = default_result () in
+          Lwt.return
+            {default_result with
+               res_content_length = Some (Int64.of_int (String.length x));
+               res_content_type = Some "text/html";
+               res_etag = md5;
+               res_headers= Http_headers.dyn_headers;
+               res_stream =
+                (Ocsigen_stream.make
+                   (fun () -> Ocsigen_stream.cont x
+                      (fun () -> Ocsigen_stream.empty None)),
+                 None)
+           }
+
+      end
+
+
+    module Contreg_ = struct
+      open XHTML.M
+      open Xhtmltypes
+
+      type page = T.content
+
+      type options = unit
+
+      let send ?options ?(cookies=[]) ?charset ?code
+          ?content_type ?headers ~sp content =
+        Cont_content.result_of_content content >>= fun r ->
+        Lwt.return
+          {r with
+             res_cookies=
+              Eliom_services.cookie_table_of_eliom_cookies ~sp cookies;
+             res_code= code_of_code_option code;
+             res_charset= Some "utf-8" 
+              (* For Eliom_duce, we impose utf-8 *);
+             res_content_type= (match content_type with
+                                  | None -> r.res_content_type
+                                  | _ -> content_type
+                               );
+             res_headers= (match headers with
+                             | None -> r.res_headers
+                             | Some headers -> 
+                                 Http_headers.with_defaults
+                                   headers r.res_headers
+                          );
+          }
+
+    end
+
+    module Contreg = Eliom_mkreg.MakeRegister(Contreg_)
+
+    include Xhtmlforms
+    include Contreg
+
+
+   end : XhtmlSig with type page = T.content
+)
 
 
 module Blocks = SubXhtml(struct
