@@ -25,14 +25,29 @@ open Eliom_services
 open Xhtmltypes_duce
 open Eliom_tools_common
 
-let attrib_list (s: string list): string = String.concat " " s
 
-let menu ?(classe=[]) first l ?service:current ~sp =
+let same_service_opt sp s sopt =
+  let same_url url = make_uri ~absolute_path:true ~sp ~service:s () = url in
+  match sopt with
+    | None -> (* MAYBE : use this or get_original_full_path_string *)
+        same_url ("/" ^ Eliom_sessions.get_current_full_path_string sp)
+    | Some s' -> same_url (make_uri ~absolute_path:true ~service:s' ~sp ())
+
+
+let attrib_list (s: string list): string = String.concat " " s
+let ul_attribs classes id level =
+  {{ { class={: attrib_list classes :} } ++
+     {{ match id, level with
+          | Some id, 0 -> {{ { id={{Ocamlduce.Utf8.make id}} } }}
+          | _ -> {{ {} }}
+     }} }}
+
+let menu ?(classe=[]) ?id first l ?service:current ~sp =
   let rec aux = function
     | [] -> []
     | [(url, text)] ->
         let classe = [last_class] in
-        if Some url = (* problem with preapplied services with == *) current
+        if same_service_opt sp url current
         then
           [{{ <li class={: attrib_list (current_class::classe) :}>{: text :} }}]
             (* [li ~a:[a_class (current_class::classe)] text] *)
@@ -40,7 +55,7 @@ let menu ?(classe=[]) first l ?service:current ~sp =
           [{{ <li class={: attrib_list classe :}>[{: a url sp {: text :} () :}] }}]
             (* [li ~a:[a_class classe] [a url sp text ()] *)
     | (url, text)::l ->
-        (if Some url = (* problem with preapplied services with == *) current
+        (if same_service_opt sp url current
          then
            {{ <li class={: current_class :}>{: text :} }}
              (* (li ~a:[a_class [current_class]] text) *)
@@ -50,28 +65,28 @@ let menu ?(classe=[]) first l ?service:current ~sp =
   in match first::l with
     | [] -> assert false
     | [(url, text)] ->
-        {{ <ul class={: attrib_list (menu_class::classe) :}>
+        {{ <ul ({{ ul_attribs (menu_class::classe) id 0 }})>
              [{:
                let liclasse = [first_class; last_class] in
-                 if Some url = current then
+                 if same_service_opt sp url current then
                    {{ <li class={: attrib_list (current_class::liclasse) :}>{: text :} }}
                  else
                    {{ <li class={: attrib_list liclasse :}>[{: a url sp {: text :} () :}] }} :}] }}
     | (url, text)::l ->
-        {{ <ul class={: attrib_list (menu_class::classe) :}>
+        {{ <ul ({{ ul_attribs (menu_class::classe) id 0 }})>
              [{:
                let liclasse = [first_class] in
-                 if Some url = current then
+                 if same_service_opt sp url current then
                    {{ <li class={: attrib_list (current_class::liclasse) :}>{: text :} }}
                  else
                    {{ <li class={: attrib_list liclasse :}>[{: a url sp {: text :} () :}] }} :}
                 !{: aux l :}] }}
 
 
-let find_in_hierarchy service (main, pages) =
+let find_in_hierarchy sp service (main, pages) =
   let rec aux service i = function
     | [] -> raise Not_found
-    | (_, Site_tree (Main_page s, hsl))::_ when s = service ->
+    | (_, Site_tree (Main_page s, hsl))::_ when same_service_opt sp s service ->
         (try
           i::aux service 0 hsl
         with Not_found -> [i])
@@ -81,15 +96,13 @@ let find_in_hierarchy service (main, pages) =
           i::aux service 0 hsl
         with Not_found -> aux service (i+1) l)
   in
-  try
-    match service with
-      | None -> []
-      | Some service -> aux service 0 pages
+  try aux service 0 pages
   with Not_found -> []
 
 
 let hierarchical_menu_depth_first
     ?(classe=[])
+    ?id
     ?(whole_tree=false)
     ((page, pages) as the_menu)
     ?service
@@ -163,14 +176,16 @@ let hierarchical_menu_depth_first
     let classe = (level_class^string_of_int level)::classe in
     match one_menu true 0 pages with
     | {{ [] }} -> {{ [] }}
-    | {{ l }} -> {{ [ <ul class={: attrib_list (menu_class::classe) :}>l ] }}
+    | {{ l }} ->
+        {{ [ <ul ({{ ul_attribs (menu_class::classe) id level }})>l ] }}
   in
 
-  (depth_first_fun pages 0 (find_in_hierarchy service the_menu))
+  (depth_first_fun pages 0 (find_in_hierarchy sp service the_menu))
 
 
 let hierarchical_menu_breadth_first
     ?(classe=[])
+    ?id
     (((page, pages): ([< Eliom_services.get_service_kind],
                 [< Eliom_services.registrable ],
                 Eliom_duce.Blocks.a_content_elt_list) hierarchical_site) as the_menu)
@@ -228,10 +243,11 @@ let hierarchical_menu_breadth_first
     in
     match one_menu true 0 pages with
     | [] -> l
-    | li::lis -> {{ [ <ul class={: attrib_list (menu_class::classe) :}>[ {: li :} !{: lis :} ] !l ] }}
+    | li::lis ->
+        {{ [ <ul ({{ ul_attribs (menu_class::classe) id level}})>[ {: li :} !{: lis :} ] !l ] }}
 
   in
-  (breadth_first_fun pages 0 (find_in_hierarchy service the_menu))
+  (breadth_first_fun pages 0 (find_in_hierarchy sp service the_menu))
 
 
 let structure_links (default, pages) ?service ~sp =
@@ -254,12 +270,12 @@ let structure_links (default, pages) ?service ~sp =
   in
   let rec create_rev parent = function
   | [] -> raise Not_found
-  | (_, (Site_tree (Main_page s, [])))::l when Some s = service ->
+  | (_, (Site_tree (Main_page s, [])))::l when same_service_opt sp s service ->
       make_rev parent []
   | (_, Disabled)::l
   | (_, Site_tree (_, []))::l -> create_rev parent l
   | (_, Site_tree (Main_page page, hsl))::l ->
-      if Some page = service 
+      if same_service_opt sp page service
       then make_rev parent (List.fold_left make_rels [] hsl)
       else
         (try create_rev (Some page) hsl
@@ -271,7 +287,7 @@ let structure_links (default, pages) ?service ~sp =
   try
     match default with
     | Main_page def ->
-        if Some def = service then
+        if same_service_opt sp def service then
           {{ [ !{: List.fold_left make_rels [] pages :} ] }}
         else create_rev (Some def) pages
     | _ ->
