@@ -50,6 +50,23 @@ open Ocsigen_extensions
    like "close all opened sessions" for one user, or limitation of
    the number of sessions one user can open concurrently, or setting
    data for one group of sessions.
+
+   The default duration of session may be set in Ocsigen's configuration file,
+   as options for the Eliom module. Each Eliom site can override these options
+   using functions of this module.
+
+   Setting sessions timeout in the configuration file. Example:
+    [
+      <extension findlib-package="ocsigen.ext.eliom">
+        <datatimeout value="3600"> (* in memory data sessions *)
+        <persistenttimeout value="infinity"> (* persistent session data *)
+        <servicetimeout value="3600"> (* session services *)
+        <volatiletimeout value="3600"> (* both session services and
+                                          in memory data sessions *)
+      </extension>
+    ]
+
+
   *)
 
 
@@ -79,14 +96,14 @@ val get_remote_inet_addr : sp:server_params -> Unix.inet_addr
 (** returns the full path of the URL as a string. *)
 val get_current_full_path_string : sp:server_params -> string
 
-(** returns the full path of the URL using the type {!Ocsigen_extensions.url_path} *)
-val get_current_full_path : sp:server_params -> url_path
+(** returns the full path of the URL using the type {!Ocsigen_lib.url_path} *)
+val get_current_full_path : sp:server_params -> Ocsigen_lib.url_path
 
 (** returns the full path of the URL as first sent by the browser (not changed by previous extensions like rewritemod) *)
 val get_original_full_path_string : sp:server_params -> string
 
 (** returns the full path of the URL as first sent by the browser (not changed by previous extensions like rewritemod) *)
-val get_original_full_path : sp:server_params -> url_path
+val get_original_full_path : sp:server_params -> Ocsigen_lib.url_path
 
 (** returns the sub path of the URL as a string.
     The sub-path is the full path without the path of the site (set in the
@@ -94,11 +111,11 @@ val get_original_full_path : sp:server_params -> url_path
  *)
 val get_current_sub_path_string : sp:server_params -> string
 
-(** returns the sub path of the URL using the type {!Ocsigen_extensions.url_path}.
+(** returns the sub path of the URL using the type {!Ocsigen_lib.url_path}.
     The sub-path is the full path without the path of the site (set in the
     configuration file).
  *)
-val get_current_sub_path : sp:server_params -> url_path
+val get_current_sub_path : sp:server_params -> Ocsigen_lib.url_path
 
 (** returns the hostname that has been sent by the user agent.
     For HTTP/1.0, the Host field is not mandatory in the request.
@@ -136,10 +153,10 @@ val get_server_port : sp:server_params -> int
 val get_ssl : sp:server_params -> bool
 
 (** returns the suffix of the current URL *)
-val get_suffix : sp:server_params -> url_path
+val get_suffix : sp:server_params -> Ocsigen_lib.url_path option
 
 (** returns the cookies sent by the browser *)
-val get_cookies : sp:server_params -> string Ocsigen_http_frame.Cookievalues.t
+val get_cookies : sp:server_params -> string Ocsigen_lib.String_Table.t
 
 
 
@@ -163,6 +180,21 @@ val get_cookies : sp:server_params -> string Ocsigen_http_frame.Cookievalues.t
     Sessions whose timeout has been set individually with
     {!Eliom_sessions.set_volatile_session_timeout} won't be affected.
 
+    If [~session_name] is not present, it is the default for all session names,
+    and in that case [recompute_expdates] is ignored. [~session_name:None]
+    means the default session name.
+
+    If [~override_configfile] is [true] (default ([false]),
+    then the function will set the timeout even if it has been
+    modified in the configuration file.
+    It means that by default, these functions have no effect
+    if there is a value in the configuration file.
+    This gives the ability to override the values choosen by the module
+    in the configuration file.
+    Use [~override_configfile:true] for example if your
+    Eliom module wants to change the values afterwards
+    (for example in the site configuration Web interface).
+
     {e Warning: If you use one of these functions after the
     initialisation phase, you must give the [~sp] parameter, otherwise
     it will raise the exception
@@ -174,31 +206,44 @@ val get_cookies : sp:server_params -> string Ocsigen_http_frame.Cookievalues.t
     service session and volatile data session) (server side).
 *)
 val set_global_volatile_session_timeout :
-  ?session_name:string -> 
+  ?session_name:string option -> 
   ?sp:server_params ->
-  ?recompute_expdates:bool -> float option -> unit
+  ?recompute_expdates:bool -> 
+  ?override_configfile:bool ->
+  float option -> unit
 
 (** Sets the timeout for service sessions (server side).
 *)
 val set_global_service_session_timeout :
-  ?session_name:string -> 
+  ?session_name:string option -> 
   ?sp:server_params ->
-  ?recompute_expdates:bool -> float option -> unit
+  ?recompute_expdates:bool -> 
+  ?override_configfile:bool ->
+  float option -> unit
 
 (** Sets the timeout for volatile (= "in memory") data sessions (server side).
 *)
 val set_global_volatile_data_session_timeout :
-  ?session_name:string -> 
+  ?session_name:string option -> 
   ?sp:server_params ->
-  ?recompute_expdates:bool -> float option -> unit
+  ?recompute_expdates:bool -> 
+  ?override_configfile:bool ->
+  float option -> unit
 
 (** Sets the timeout for persistent sessions (server side).
 *)
-val set_global_persistent_data_session_timeout : ?session_name:string ->
+val set_global_persistent_data_session_timeout :
+  ?session_name:string option ->
   ?sp:server_params -> 
   ?recompute_expdates:bool ->
+  ?override_configfile:bool ->
   float option -> unit
 
+
+
+
+
+
 (** Returns the timeout for service sessions (server side).
 *)
 val get_global_service_session_timeout :
@@ -332,7 +377,7 @@ type 'a session_data =
     https, the unsecure one in http.
 *)
 val set_service_session_group :
-  ?set_max: int option ->
+  ?set_max: int ->
   ?session_name:string ->
   ?secure:bool ->
   sp:server_params ->
@@ -341,6 +386,7 @@ val set_service_session_group :
 
 (** Remove the session from its group *)
 val unset_service_session_group :
+  ?set_max: int ->
   ?session_name:string ->
   ?secure:bool ->
   sp:server_params ->
@@ -363,7 +409,7 @@ val get_service_session_group :
     number of sessions in the group. [None] means "no limitation".
 *)
 val set_volatile_data_session_group :
-  ?set_max: int option ->
+  ?set_max: int ->
   ?session_name:string ->
   ?secure:bool ->
   sp:server_params ->
@@ -372,6 +418,7 @@ val set_volatile_data_session_group :
 
 (** Remove the session from its group *)
 val unset_volatile_data_session_group :
+  ?set_max: int ->
   ?session_name:string ->
   ?secure:bool ->
   sp:server_params ->
@@ -426,25 +473,84 @@ val get_persistent_data_session_group :
     value will be used only as default value if you do not specify the
     optional parameter [?set_max] of function
     {!Eliom_sessions.set_volatile_data_session_group}.
+
+    If there is no group, the number of sessions is limitated by sub network
+    (which can be a problem for example if the server is behind a
+    reverse proxy).
+    It is highly recommended to use session groups!
 *)
 
 (** Sets the maximum number of service sessions in a session group
     (see above).
 *)
 val set_default_max_service_sessions_per_group :
-  sp:server_params -> int option -> unit
+  ?sp:server_params -> ?override_configfile:bool -> int -> unit
 
 (** Sets the maximum number of volatile data sessions in a session
     group (see above).
 *)
 val set_default_max_volatile_data_sessions_per_group :
-  sp:server_params -> int option -> unit
+  ?sp:server_params -> ?override_configfile:bool -> int -> unit
 
 (** Sets the maximum number of persistent data sessions in a session
     group (see above).
 *)
 val set_default_max_persistent_data_sessions_per_group :
-  sp:server_params -> int option -> unit
+  ?sp:server_params -> ?override_configfile:bool -> int option -> unit
+
+(** Sets the maximum number of service sessions in a subnet (see above).
+*)
+val set_default_max_service_sessions_per_subnet :
+  ?sp:server_params -> ?override_configfile:bool -> int -> unit
+
+(** Sets the maximum number of volatile data sessions in a subnet (see above).
+*)
+val set_default_max_volatile_data_sessions_per_subnet :
+  ?sp:server_params -> ?override_configfile:bool -> int -> unit
+
+(** Sets the maximum number of volatile sessions (data and service) in a session
+    group (see above).
+*)
+val set_default_max_volatile_sessions_per_group :
+  ?sp:server_params -> ?override_configfile:bool -> int -> unit
+
+(** Sets the maximum number of volatile sessions (data and service) 
+    in a subnet (see above).
+*)
+val set_default_max_volatile_sessions_per_subnet :
+  ?sp:server_params -> ?override_configfile:bool -> int -> unit
+
+(** Sets the mask for subnet (IPV4). *)
+val set_ipv4_subnet_mask :
+  ?sp:server_params -> ?override_configfile:bool -> int32 -> unit
+
+(** Sets the mask for subnet (IPV6). *)
+val set_ipv6_subnet_mask :
+  ?sp:server_params -> ?override_configfile:bool -> int64 * int64 -> unit
+
+
+
+(** Sets the maximum number of service sessions in the current session
+    group (or for the client sub network, if there is no group).
+*)
+val set_max_service_sessions_for_group_or_subnet :
+  ?session_name:string ->
+  ?secure:bool ->
+  sp:server_params ->
+  int ->
+  unit
+
+
+(** Sets the maximum number of volatile data sessions in the current session
+    group (or for the client sub network, if there is no group).
+*)
+val set_max_volatile_data_sessions_for_group_or_subnet :
+  ?session_name:string ->
+  ?secure:bool ->
+  sp:server_params ->
+  int ->
+  unit
+
 
 (** {3 Cookies} *)
 
@@ -498,11 +604,26 @@ val set_persistent_data_session_cookie_exp_date :
 
 (** {3 Exceptions and fallbacks} *)
 
-(** returns the exceptions that have been sent by the previous services
-   to their fallback, if any. Keep an eye on these exception to know what
-   succeeded before that service was called (failed connection, timeout ...)
+(** returns a table in which you can store all the data you want during a
+    request. It can also be used to send information after an action.
+    Keep an eye on this information to know what
+    succeeded before the current service was called
+    (failed connection, timeout ...)
+    The table is created at the beginning of the request.
  *)
-val get_exn : sp:server_params -> exn list
+val get_request_cache : sp:server_params -> Polytables.t
+
+(** Remove all data from the request cache *)
+val clean_request_cache : sp:server_params -> unit
+
+(** returns [true] if the coservice called has not been found.
+    In that case, the current service is the fallback.
+*)
+val get_link_too_old : sp:server_params -> bool
+
+(** returns the list of names of service sessions expired for the current 
+    request. *)
+val get_expired_service_sessions : sp:server_params -> string list
 
 (** returns the HTTP error code sent by the Ocsigen extension
    that tried to answer to the request before Eliom.
@@ -526,13 +647,13 @@ val get_previous_extension_error_code : sp:server_params -> int
  *)
 
 (** returns the filename used by Ocsigen for the uploaded file. *)
-val get_tmp_filename : file_info -> string
+val get_tmp_filename : Ocsigen_lib.file_info -> string
 
 (** returns the size of the file. *)
-val get_filesize : file_info -> int64
+val get_filesize : Ocsigen_lib.file_info -> int64
 
 (** returns the name the file had on the client when it has been sent. *)
-val get_original_filename : file_info -> string
+val get_original_filename : Ocsigen_lib.file_info -> string
 
 
 
@@ -554,7 +675,7 @@ val get_original_filename : file_info -> string
 val get_config : unit -> Simplexmlparser.xml list
 
 (** returns the root of the site. *)
-val get_site_dir : sp:server_params -> url_path
+val get_site_dir : sp:server_params -> Ocsigen_lib.url_path
 
 (** returns the default charset for this site *)
 val get_config_default_charset : sp:server_params -> string
@@ -662,6 +783,10 @@ val remove_persistent_session_data :
 
     By default will close both secure and unsecure sessions, but
     if [~secure] is present.
+
+    Warning: you may also want to remove some data from the polymorphic
+    request data table when closing a session 
+    (See {!Eliom_sessions.get_request_cache}).
 *)
 val close_session :
   ?close_group:bool ->
@@ -962,6 +1087,18 @@ val get_initial_get_params : sp:server_params -> (string * string) list
    that do not concern the running service. *)
 val get_other_get_params : sp:server_params -> (string * string) list
 
+(** returns non localized parameters in the URL. *)
+val get_nl_get_params : 
+  sp:server_params -> (string * string) list Ocsigen_lib.String_Table.t
+
+(** returns persistent non localized parameters in the URL. *)
+val get_persistent_nl_get_params : 
+  sp:server_params -> (string * string) list Ocsigen_lib.String_Table.t
+
+(** returns non localized POST parameters. *)
+val get_nl_post_params : 
+  sp:server_params -> (string * string) list Ocsigen_lib.String_Table.t
+
 (** returns the parameters in the body of the HTTP request (POST parameters)
    that concern the running service *)
 val get_post_params : sp:server_params -> (string * string) list Lwt.t
@@ -971,6 +1108,7 @@ val get_post_params : sp:server_params -> (string * string) list Lwt.t
 val get_all_post_params : sp:server_params -> (string * string) list
 
 
+(*
 (** {2 Default timeouts} *)
 
 (** returns the default timeout for service sessions (server side).
@@ -989,25 +1127,6 @@ val get_default_service_session_timeout : unit -> float option
     *)
 val get_default_volatile_data_session_timeout : unit -> float option
 
-
-(** sets the default timeout for volatile (= "in memory")
-   sessions (i.e. both service session and volatile data session)
-   (server side).
-   [None] = no timeout.
-    *)
-val set_default_volatile_session_timeout : float option -> unit
-
-(** sets the default timeout for service sessions.
-    [None] = no timeout.
-    *)
-val set_default_service_session_timeout : float option -> unit
-
-(** sets the default timeout for "volatile data" sessions (server side).
-    [None] = no timeout.
-    *)
-val set_default_volatile_data_session_timeout : float option -> unit
-
-
 (** returns the default timeout for sessions (server side).
     The default timeout is common for all sessions for which no other value
     has been set. At the beginning of the server, it is taken from the
@@ -1016,10 +1135,40 @@ val set_default_volatile_data_session_timeout : float option -> unit
     *)
 val get_default_persistent_data_session_timeout : unit -> float option
 
+(** sets the default timeout for volatile (= "in memory")
+   sessions (i.e. both service session and volatile data session)
+   (server side).
+   [None] = no timeout.
+
+   Warning: this function sets the default for all sites. You should
+   probably use [set_global_volatile_session_timeout] instead.
+    *)
+val set_default_volatile_session_timeout : float option -> unit
+
+(** sets the default timeout for service sessions.
+    [None] = no timeout.
+
+    Warning: this function sets the default for all sites. You should
+    probably use [set_global_service_session_timeout] instead.
+    *)
+val set_default_service_session_timeout : float option -> unit
+
+(** sets the default timeout for "volatile data" sessions (server side).
+    [None] = no timeout.
+
+    Warning: this function sets the default for all sites. You should
+    probably use [set_global_volatile_data_session_timeout] instead.
+    *)
+val set_default_volatile_data_session_timeout : float option -> unit
+
 (** sets the default timeout for sessions (server side).
     [None] = no timeout.
+
+    Warning: this function sets the default for all sites. You should
+    probably use [set_global_persistent_data_session_timeout] instead.
     *)
 val set_default_persistent_data_session_timeout : float option -> unit
+*)
 
 
 
@@ -1102,6 +1251,13 @@ val get_session_service_table :
   unit ->
   Eliom_common.tables ref
 
+val get_session_service_table_if_exists :
+  ?session_name:string -> 
+  ?secure:bool ->
+  sp:server_params -> 
+  unit ->
+  Eliom_common.tables ref
+
 val get_sitedata : sp:server_params -> Eliom_common.sitedata
 
 (*
@@ -1124,17 +1280,17 @@ val get_persistent_cookie_exp_date : ?session_name:string ->
 (** returns the values of the Eliom's cookies for persistent sessions
    sent by the browser. *)
 val get_persistent_cookies :
-  sp:server_params -> string Ocsigen_http_frame.Cookievalues.t
+  sp:server_params -> string Ocsigen_lib.String_Table.t
 
 (** returns the values of Eliom's cookies for non persistent sessions
    sent by the browser. *)
 val get_data_cookies :
-    sp:server_params -> string Ocsigen_http_frame.Cookievalues.t
+    sp:server_params -> string Ocsigen_lib.String_Table.t
 
 val find_sitedata : string -> server_params option -> Eliom_common.sitedata
 
 val set_site_handler : Eliom_common.sitedata ->
-  (server_params -> exn -> Eliom_common.result_to_send Lwt.t) -> unit
+  (server_params -> exn -> Ocsigen_http_frame.result Lwt.t) -> unit
 
 
 (** Returns the http error code of the request before Eliom was called *)
@@ -1144,3 +1300,7 @@ val get_previous_extension_error_code :sp:server_params -> int
 
 val sp_of_esp : Eliom_common.server_params -> server_params
 val esp_of_sp : server_params -> Eliom_common.server_params
+
+
+(**/**)
+val get_si :sp:server_params -> Eliom_common.sess_info
