@@ -29,6 +29,7 @@ open Eliom_tools_common
 
 val menu :
   ?classe:XHTML.M.nmtoken list ->
+  ?id:string ->
   (([< get_service_kind ] as 'a, [< registrable ] as 'b) one_page *
      Xhtmltypes.a_content XHTML.M.elt list)
   ->
@@ -48,6 +49,9 @@ val menu :
      (infos, <:xmllist< More infos >>)
    ] current sp]
 
+   The [service] argument is used to find which item(s) to highlight. If
+   service is [None], the current url is used.
+
 *)
 
 (** {2 Hierchical sites } *)
@@ -62,12 +66,14 @@ val menu :
     a depth-first algorithm. It means that the first menu item will be
     displayed, followed by the whole sub-menu for this item, then the second
     menu item with its sub-menu, and so on.
-    By default, only the sub-menus corresponding to the current page
-    are displayed. If you want all the sub-menus to be displayed, specify
-    [?whole_tree=true].
+    By default, only the sub-menus for to the url corresponding to
+    the argument [service] are displayed. If you want all the sub-menus to be
+    displayed, specify [?whole_tree=true]. If [service] is [None], the current
+    page is used.
  *)
 val hierarchical_menu_depth_first :
   ?classe:XHTML.M.nmtoken list ->
+  ?id:string ->
   ?whole_tree:bool ->
   ([< Eliom_services.get_service_kind ] as 'a,
    [< Eliom_services.registrable ] as 'b,
@@ -87,11 +93,13 @@ val hierarchical_menu_depth_first :
     The menu is constructed by exploring the tree using
     a breadth_first algorithm. It means that the whole menu for one
     level will be displayed, followed by all sub-menus.
-    Only the sub-menu corresponding to the current page
-    is displayed.
+
+    Only the sub-menus for to the url corresponding to the argument [service]
+    are displayed. If [service] is [None], the current url is used.
  *)
 val hierarchical_menu_breadth_first :
   ?classe:XHTML.M.nmtoken list ->
+  ?id:string ->
   ([< Eliom_services.get_service_kind ] as 'a,
    [< Eliom_services.registrable ] as 'b,
    Xhtmltypes.a_content XHTML.M.elt list)
