@@ -27,71 +27,98 @@ open Ocsigen_extensions
 
 let add_naservice_table at (key, elt) =
   match at with
-  | Eliom_common.AVide ->
-      Eliom_common.ATable
-        (Eliom_common.NAserv_Table.add
-           key elt Eliom_common.NAserv_Table.empty)
-  | Eliom_common.ATable t ->
-      Eliom_common.ATable (Eliom_common.NAserv_Table.add key elt t)
-
+    | Eliom_common.AVide ->
+        Eliom_common.ATable
+          (Eliom_common.NAserv_Table.add
+             key elt Eliom_common.NAserv_Table.empty)
+    | Eliom_common.ATable t ->
+        Eliom_common.ATable (Eliom_common.NAserv_Table.add key elt t)
+          
 let find_naservice_table at k =
   match at with
-  | Eliom_common.AVide -> raise Not_found
-  | Eliom_common.ATable t -> Eliom_common.NAserv_Table.find k t
-
-let remove_naservice_table at k =
-  match at with
-  | Eliom_common.AVide -> Eliom_common.AVide
-  | Eliom_common.ATable t ->
-      Eliom_common.ATable (Eliom_common.NAserv_Table.remove k t)
+    | Eliom_common.AVide -> raise Not_found
+    | Eliom_common.ATable t -> Eliom_common.NAserv_Table.find k t
 
 let add_naservice
-    (_, naservicetableref, _, containstimeouts) duringsession name
+    tables
+    ?sp name
     (max_use, expdate, naservice) =
   let generation = Ocsigen_extensions.get_numberofreloads () in
-  (if not duringsession
+  (if (sp = None) (* not duringsession *)
   then
     try
-      let (g, _, _, _) = find_naservice_table !naservicetableref name in
+      let (g, _, _, _, _) = 
+        find_naservice_table !(tables.Eliom_common.table_naservices) name 
+      in
       if g = generation then
         match name with
-        | Eliom_common.Na_no
-        | Eliom_common.Na_get' _
-        | Eliom_common.Na_post' _ ->
+        | Eliom_common.SNa_no
+        | Eliom_common.SNa_get' _
+        | Eliom_common.SNa_post' _ ->
             raise (Eliom_common.Eliom_duplicate_registration
                      "<non-attached coservice>")
-        | Eliom_common.Na_get_ n ->
+        | Eliom_common.SNa_get_ n ->
             raise (Eliom_common.Eliom_duplicate_registration
                      ("GET non-attached service "^n))
-        | Eliom_common.Na_post_ n ->
+        | Eliom_common.SNa_post_ n ->
             raise (Eliom_common.Eliom_duplicate_registration
                      ("POST non-attached service "^n))
-        | Eliom_common.Na_void_dontkeep
-        | Eliom_common.Na_void_keep ->
+        | Eliom_common.SNa_void_dontkeep
+        | Eliom_common.SNa_void_keep ->
             raise (Eliom_common.Eliom_duplicate_registration
                      "<void coservice>")
+        | Eliom_common.SNa_get_csrf_safe _
+        | Eliom_common.SNa_post_csrf_safe _
+          -> assert false
     with Not_found -> ());
 
   (match expdate with
-  | Some _ -> containstimeouts := true
+  | Some _ -> tables.Eliom_common.table_contains_naservices_with_timeout
+      <- true
   | _ -> ());
 
-  naservicetableref :=
-    add_naservice_table !naservicetableref
-      (name, (generation, max_use, expdate, naservice))
+  let node = match name with
+    | Eliom_common.SNa_get' _ | Eliom_common.SNa_post' _ ->
+        Some 
+          (tables.Eliom_common.service_dlist_add ?sp (Ocsigen_lib.Right name))
+    | _ -> None
+  in
 
-let remove_naservice (_, atr, _, _) name =
-  atr := remove_naservice_table !atr name
+  tables.Eliom_common.table_naservices :=
+    add_naservice_table !(tables.Eliom_common.table_naservices)
+      (name, (generation, max_use, expdate, naservice, node))
 
-let find_naservice now ((_, atr, _, _) as str) name =
-  let ((_, _, expdate, _) as p) = find_naservice_table !atr name in
+let remove_naservice_ tables name nodeopt =
+  match nodeopt with
+    | None ->
+        tables.Eliom_common.table_naservices :=
+          Eliom_common.remove_naservice_table
+            !(tables.Eliom_common.table_naservices) name
+    | Some node ->
+        Ocsigen_cache.Dlist.remove node
+
+let find_naservice now tables name =
+  let ((_, _, expdate, _, nodeopt) as p) = 
+    find_naservice_table !(tables.Eliom_common.table_naservices) name
+  in
   match expdate with
-  | Some (_, e) when !e < now ->
-      (* Service expired. Removing it. *)
-      Ocsigen_messages.debug2 "--Eliom: Non attached service expired. I'm removing it";
-      remove_naservice str name;
-      raise Not_found
-  | _ -> p
+    | Some (_, e) when !e < now ->
+        (* Service expired. Removing it. *)
+        Ocsigen_messages.debug2
+          "--Eliom: Non attached service expired. I'm removing it";
+        remove_naservice_ tables name nodeopt;
+        raise Not_found
+    | _ -> 
+        (match nodeopt with
+           | Some node -> Ocsigen_cache.Dlist.up node
+           | None -> ());
+        p
+
+let remove_naservice tables name =
+  let (_, _, _, _, nodeopt) = 
+    find_naservice_table !(tables.Eliom_common.table_naservices) name
+  in
+  remove_naservice_ tables name nodeopt
 
 
 (******************************************************************)
@@ -100,14 +127,13 @@ let make_naservice
     now
     (ri,
      si,
-     cookies_to_set,
      (((service_cookies_info, _, _), secure_ci) as all_cookie_info))
     sitedata
     =
 
   let rec find_aux sci =
     match
-      Ocsigen_http_frame.Cookievalues.fold
+      Ocsigen_lib.String_Table.fold
         (fun fullsessname (_, r) beg ->
           match beg with
           | Eliom_common.Found _ -> beg
@@ -120,9 +146,10 @@ let make_naservice
                     Eliom_common.Found
                       ((find_naservice
                           now !(c.Eliom_common.sc_table)
-                          si.Eliom_common.si_nonatt_info),
+                          (Eliom_common.na_key_serv_of_req 
+                             si.Eliom_common.si_nonatt_info)),
                        !(c.Eliom_common.sc_table),
-                       Some fullsessname)
+                        Some fullsessname)
                   with Not_found -> beg
         )
         sci
@@ -131,7 +158,7 @@ let make_naservice
     | Eliom_common.Found v -> v
     | Eliom_common.Notfound _ ->
         (find_naservice now sitedata.Eliom_common.global_services
-           si.Eliom_common.si_nonatt_info,
+           (Eliom_common.na_key_serv_of_req si.Eliom_common.si_nonatt_info),
          sitedata.Eliom_common.global_services,
          None)
   in
@@ -156,57 +183,57 @@ let make_naservice
          We call the same URL without non-attached parameters.
        *)
       match si.Eliom_common.si_nonatt_info with
-      | Eliom_common.Na_void_keep (* ? *)
-      | Eliom_common.Na_void_dontkeep (* ? *)
-      | Eliom_common.Na_no -> assert false
-      | Eliom_common.Na_post_ _
-      | Eliom_common.Na_post' _ ->
+      | Eliom_common.RNa_no -> assert false
+      | Eliom_common.RNa_post_ _
+      | Eliom_common.RNa_post' _ ->
 (*VVV (Some, Some) or (_, Some)? *)
           Ocsigen_messages.debug2
             "--Eliom: Link too old to a non-attached POST coservice. I will try without POST parameters:";
-          Eliom_common.change_request_info
+          Polytables.set 
+            ri.request_info.ri_request_cache
+            Eliom_common.eliom_link_too_old
+            true;
+          Eliom_common.get_session_info
             {ri with Ocsigen_extensions.request_info =
                 { ri.Ocsigen_extensions.request_info with
                     Ocsigen_extensions.ri_get_params =
                       lazy si.Eliom_common.si_other_get_params;
-                    ri_post_params = lazy (return []);
+                    ri_post_params = (fun _ -> return []);
+                    ri_files = (fun _ -> Lwt.return []);
                     ri_method = Ocsigen_http_frame.Http_header.GET;
-                    ri_extension_info =
-                      Eliom_common.Eliom_Link_too_old ::
-                      ri.Ocsigen_extensions.request_info.Ocsigen_extensions.ri_extension_info
             }}
             si.Eliom_common.si_previous_extension_error
           >>=
             (fun (ri', si') ->
                fail (Eliom_common.Eliom_retry_with (ri', 
                                                     si',
-                                                    cookies_to_set,
                                                     all_cookie_info)))
 
-      | Eliom_common.Na_get_ _
-      | Eliom_common.Na_get' _ ->
+      | Eliom_common.RNa_get_ _
+      | Eliom_common.RNa_get' _ ->
           Ocsigen_messages.debug2
             "--Eliom: Link too old. I will try without non-attached parameters:";
-          Eliom_common.change_request_info
+          Polytables.set 
+            ri.request_info.ri_request_cache
+            Eliom_common.eliom_link_too_old
+            true;
+          Eliom_common.get_session_info
             {ri with request_info =
                 { ri.request_info with
                     ri_get_params =
                       lazy si.Eliom_common.si_other_get_params;
-                    ri_post_params = lazy (return []);
+                    ri_post_params = (fun _ -> return []);
+                    ri_files = (fun _ -> Lwt.return []);
                     ri_method = Ocsigen_http_frame.Http_header.GET;
-                    ri_extension_info =
-                      Eliom_common.Eliom_Link_too_old::
-                        ri.request_info.ri_extension_info
                 }
            }
             si.Eliom_common.si_previous_extension_error
             >>=
           (fun (ri', si') ->
             fail (Eliom_common.Eliom_retry_with (ri', si',
-                                                      cookies_to_set,
-                                                      all_cookie_info)))
+                                                 all_cookie_info)))
   ) >>=
-  (fun ((_, max_use, expdate, naservice),
+  (fun ((_, max_use, expdate, naservice, node), 
         tablewhereithasbeenfound,
         fullsessname) ->
     (naservice
@@ -214,7 +241,7 @@ let make_naservice
           sitedata
           all_cookie_info
           ri
-          []
+          None
           si
           fullsessname)) >>=
     (fun r ->
@@ -227,9 +254,11 @@ let make_naservice
       | None -> ()
       | Some r ->
           if !r = 1
-          then
-            remove_naservice tablewhereithasbeenfound
-              si.Eliom_common.si_nonatt_info
+          then 
+            remove_naservice_
+              tablewhereithasbeenfound 
+              (Eliom_common.na_key_serv_of_req si.Eliom_common.si_nonatt_info)
+              node
           else r := !r - 1);
       return r))
 
