@@ -52,9 +52,9 @@ let number_of_persistent_tables () =
 let number_of_persistent_table_elements () =
   List.fold_left
     (fun thr t ->
-      thr >>=
-      (fun l -> Ocsipersist.length (Ocsipersist.open_table t) >>=
-        (fun e -> return ((t, e)::l)))) (return []) !perstables
+      thr >>= fun l -> 
+      Ocsipersist.length (Ocsipersist.open_table t) >>= fun e -> 
+      return ((t, e)::l)) (return []) !perstables
 
 (* close a persistent session by cookie *)
 let close_persistent_session2 fullsessgrp cookie =
@@ -71,9 +71,12 @@ let close_persistent_session2 fullsessgrp cookie =
 
 let close_persistent_group fullsessgrp =
 (*VVV VERIFY concurrent access *)
-  Eliommod_sessiongroups.Pers.find fullsessgrp >>= fun cooklist ->
-  Lwt_util.iter (close_persistent_session2 None) cooklist >>= fun () ->
-  Eliommod_sessiongroups.Pers.remove_group fullsessgrp
+  Lwt.catch
+    (fun () ->
+       Eliommod_sessiongroups.Pers.find fullsessgrp >>= fun cooklist ->
+       Lwt_util.iter (close_persistent_session2 None) cooklist >>= fun () ->
+       Eliommod_sessiongroups.Pers.remove_group fullsessgrp)
+    (function Not_found -> Lwt.return () | e -> Lwt.fail e)
 
 (* close current persistent session *)
 let close_persistent_session ?(close_group = false) ?session_name 
@@ -83,7 +86,7 @@ let close_persistent_session ?(close_group = false) ?session_name
       let fullsessname = Eliom_common.make_fullsessname ~sp session_name in
       let ((_, _, cookie_info), secure_ci) = sp.Eliom_common.sp_cookie_info in
       let cookie_info = compute_cookie_info secure secure_ci cookie_info in
-      Lazy.force (Ocsigen_http_frame.Cookievalues.find fullsessname !cookie_info)
+      Lazy.force (Ocsigen_lib.String_Table.find fullsessname !cookie_info)
       >>= fun (_, ior) ->
       match !ior with
       | Eliom_common.SC c ->
@@ -116,12 +119,12 @@ let rec new_persistent_cookie sitedata fullsessgrp fullsessname =
             Ocsipersist.add
               (Lazy.force persistent_cookies_table) c
               (fullsessname,
-               Some 0. (* exp on server - We'll change it later *),
+               None (* Some 0. *) (* exp on server - We'll change it later *),
                Eliom_common.TGlobal (* timeout - see global config *),
                fullsessgrp)
             >>= fun () ->
             Eliommod_sessiongroups.Pers.add
-              sitedata.Eliom_common.max_persistent_data_sessions_per_group
+              (fst sitedata.Eliom_common.max_persistent_data_sessions_per_group)
               c fullsessgrp >>= fun l ->
             Lwt_util.iter
               (close_persistent_session2 None) l >>= fun () ->
@@ -140,6 +143,7 @@ let find_or_create_persistent_cookie ?session_group ?session_name ~secure ~sp ()
   let fullsessname = Eliom_common.make_fullsessname ~sp session_name in
   let fullsessgrp =
     Eliommod_sessiongroups.make_persistent_full_group_name
+      sp.Eliom_common.sp_request.Ocsigen_extensions.request_info
       sp.Eliom_common.sp_sitedata.Eliom_common.site_dir_string
       session_group
   in
@@ -147,7 +151,7 @@ let find_or_create_persistent_cookie ?session_group ?session_name ~secure ~sp ()
   let cookie_info = compute_cookie_info secure secure_ci cookie_info in
   catch
     (fun () ->
-      Lazy.force (Ocsigen_http_frame.Cookievalues.find fullsessname !cookie_info)
+      Lazy.force (Ocsigen_lib.String_Table.find fullsessname !cookie_info)
       >>= fun (old, ior) ->
       match !ior with
       | Eliom_common.SCData_session_expired
@@ -166,7 +170,7 @@ let find_or_create_persistent_cookie ?session_group ?session_name ~secure ~sp ()
             sp.Eliom_common.sp_sitedata
             fullsessgrp fullsessname >>= fun v ->
           cookie_info :=
-            Ocsigen_http_frame.Cookievalues.add
+            Ocsigen_lib.String_Table.add
               fullsessname
               (Lazy.lazy_from_val (return (None, ref (Eliom_common.SC v))))
               !cookie_info;
@@ -181,7 +185,7 @@ let find_persistent_cookie_only ?session_name ~secure ~sp () =
   let fullsessname = Eliom_common.make_fullsessname ~sp session_name in
   let ((_, _, cookie_info), secure_ci) = sp.Eliom_common.sp_cookie_info in
   let cookie_info = compute_cookie_info secure secure_ci cookie_info in
-  Lazy.force (Ocsigen_http_frame.Cookievalues.find fullsessname !cookie_info)
+  Lazy.force (Ocsigen_lib.String_Table.find fullsessname !cookie_info)
   >>= fun (_, ior) ->
   match !ior with
   | Eliom_common.SCNo_data -> raise Not_found
