@@ -1,5 +1,5 @@
 val close_service_group :
-  Eliom_common.sitedata -> Eliommod_sessiongroups.sessgrp option -> unit
+  Eliom_common.sessgrp -> unit
 val close_service_session :
   ?close_group:bool ->
   ?session_name:string -> 
@@ -7,19 +7,16 @@ val close_service_session :
   sp:Eliom_common.server_params -> unit -> unit
 val new_service_cookie :
   Eliom_common.sitedata ->
-  Eliommod_sessiongroups.sessgrp option ->
+  Eliom_common.sessgrp ->
   'a ->
-  ('a *
-   (Eliom_common.dircontent ref * Eliom_common.naservice_table ref *
-    bool ref * bool ref) *
+  ('a * Eliom_common.tables *
    float option ref * Eliom_common.timeout ref *
-   Eliommod_sessiongroups.sessgrp option ref)
+   Eliom_common.sessgrp ref * string Ocsigen_cache.Dlist.node)
   Eliom_common.SessionCookies.t ->
-  (Eliom_common.dircontent ref * Eliom_common.naservice_table ref *
-   bool ref * bool ref)
+  Eliom_common.tables
   Eliom_common.one_service_cookie_info
 val find_or_create_service_cookie :
-  ?session_group:string ->
+  ?set_session_group:string ->
   ?session_name:string ->
   secure: bool option ->
   sp:Eliom_common.server_params ->
