@@ -30,20 +30,375 @@ open Eliom_parameters
 open Eliom_sessions
 open Lwt
 
+
+(******)
+(* unregistering services *)
+let unregister_example =
+  Eliom_predefmod.Xhtml.register_new_service
+    ~path:["unregister"]
+    ~get_params:Eliom_parameters.unit
+    (fun sp () () ->
+       let s1 = Eliom_predefmod.Xhtml.register_new_service
+         ~sp
+         ~path:["unregister1"]
+         ~get_params:Eliom_parameters.unit
+         (fun sp () () -> failwith "s1")
+       in
+       let s2 = Eliom_predefmod.Xhtml.register_new_coservice
+         ~sp
+         ~fallback:s1
+         ~get_params:Eliom_parameters.unit
+         (fun sp () () -> failwith "s2")
+       in
+       let s3 = Eliom_predefmod.Xhtml.register_new_coservice'
+         ~sp
+         ~get_params:Eliom_parameters.unit
+         (fun sp () () -> failwith "s3")
+       in
+       Eliom_predefmod.Xhtml.register_for_session
+         ~sp
+         ~service:s1
+         (fun sp () () -> failwith "s4");
+       Eliom_services.unregister ~sp s1;
+       Eliom_services.unregister ~sp s2;
+       Eliom_services.unregister ~sp s3;
+       Eliom_services.unregister_for_session ~sp s1;
+       Lwt.return
+         (html
+            (head (title (pcdata "Unregistering services")) [])
+            (body [p [pcdata 
+                        "These services have been registered and unregistered"];
+                   p [a s1 sp [pcdata "regular service"] ();
+                      pcdata ", ";
+                      a s2 sp [pcdata "coservice"] ();
+                      pcdata ", ";
+                      a s3 sp [pcdata "non attached coservice"] ();
+                      pcdata ", ";
+                      a s1 sp [pcdata "session service"] ();
+                     ]]))
+    )
+
+
+(******)
+(* CSRF GET *)
+
+let csrfsafe_get_example =
+  Eliom_services.new_service
+    ~path:["csrfget"]
+    ~get_params:Eliom_parameters.unit
+    ()
+
+let csrfsafe_example_get =
+  Eliom_services.new_coservice
+    ~csrf_safe:true
+    ~timeout:10.
+    ~fallback:csrfsafe_get_example
+    ~get_params:Eliom_parameters.unit
+    ()
+
+let _ =
+  let page sp () () =
+    let l3 = Eliom_predefmod.Xhtml.get_form csrfsafe_example_get sp
+        (fun _ -> [p [Eliom_predefmod.Xhtml.string_input
+                        ~input_type:`Submit
+                        ~value:"Click" ()]])
+    in
+    return
+      (html
+       (head (title (pcdata "CSRF safe service example")) [])
+       (body [p [pcdata "A new coservice will be created each time this form is displayed"];
+              l3]))
+  in
+  Eliom_predefmod.Xhtml.register csrfsafe_get_example page;
+  Eliom_predefmod.Xhtml.register csrfsafe_example_get
+    (fun sp () () ->
+       Lwt.return
+         (html
+            (head (title (pcdata "CSRF safe service")) [])
+            (body [p [pcdata "This is a GET CSRF safe service"]])))
+
+(******)
+(* CSRF POST on CSRF GET coservice *)
+
+let csrfsafe_postget_example =
+  Eliom_services.new_service
+    ~path:["csrfpostget"]
+    ~get_params:Eliom_parameters.unit
+    ()
+
+let csrfsafe_example_post =
+  Eliom_services.new_post_coservice
+    ~csrf_safe:true
+    ~timeout:10.
+    ~fallback:csrfsafe_example_get (* !!! *)
+    ~post_params:Eliom_parameters.unit
+    ()
+
+let _ =
+  let page sp () () =
+    let l3 = Eliom_predefmod.Xhtml.post_form csrfsafe_example_post sp
+        (fun _ -> [p [Eliom_predefmod.Xhtml.string_input
+                        ~input_type:`Submit
+                        ~value:"Click" ()]]) ()
+    in
+    return
+      (html
+       (head (title (pcdata "CSRF safe service example")) [])
+       (body [p [pcdata "A new coservice will be created each time this form is displayed"];
+              l3]))
+  in
+  Eliom_predefmod.Xhtml.register csrfsafe_postget_example page;
+  Eliom_predefmod.Xhtml.register csrfsafe_example_post
+    (fun sp () () ->
+       Lwt.return
+         (html
+            (head (title (pcdata "CSRF safe service")) [])
+            (body [p [pcdata "This is a POST CSRF safe service, combined with a GET CSRF safe service"]])))
+
+
+(******)
+(* CSRF for_session *)
+
+let csrfsafe_session_example =
+  Eliom_services.new_service
+    ~path:["csrfsession"]
+    ~get_params:Eliom_parameters.unit
+    ()
+
+let csrfsafe_example_session =
+  Eliom_services.new_post_coservice'
+    ~csrf_safe:true
+    ~csrf_session_name:"plop"
+    ~csrf_secure_session:true
+    ~timeout:10.
+    ~post_params:Eliom_parameters.unit
+    ()
+
+let _ =
+  let page sp () () =
+    Eliom_predefmod.Xhtml.register_for_session
+      ~session_name:"plop"
+      ~secure:true
+      ~sp
+      ~service:csrfsafe_example_session
+      (fun sp () () ->
+         Lwt.return
+           (html
+              (head (title (pcdata "CSRF safe service")) [])
+              (body [p [pcdata "This is a POST CSRF safe service"]])));
+    let l3 = Eliom_predefmod.Xhtml.post_form csrfsafe_example_session sp
+        (fun _ -> [p [Eliom_predefmod.Xhtml.string_input
+                        ~input_type:`Submit
+                        ~value:"Click" ()]])
+        ()
+    in
+    return
+      (html
+       (head (title (pcdata "CSRF safe service example")) [])
+       (body [p [pcdata "A new coservice will be created each time this form is displayed"];
+              l3]))
+  in
+  Eliom_predefmod.Xhtml.register csrfsafe_session_example page
+
+
+
+(******)
+(* optional suffix parameters *)
+
+let optsuf =
+  register_new_service
+    ~path:["optsuf"]
+    ~get_params:(suffix(opt(string "q" ** (opt (int "i")))))
+    (fun sp o () -> 
+       Lwt.return
+         (html
+            (head (title (pcdata "")) [])
+            (body [p [pcdata (match o with 
+                                 | None -> "<none>"
+                                 | Some (s, o) -> 
+                                     s^(match o with 
+                                          | None -> "<none>"
+                                          | Some i -> string_of_int i));
+                     ]])))
+
+let optsuf2 =
+  register_new_service
+    ~path:["optsuf2"]
+    ~get_params:(suffix(opt(string "q") ** (opt (int "i"))))
+    (fun sp (s, i) () -> 
+       Lwt.return
+         (html
+            (head (title (pcdata "")) [])
+            (body [p [pcdata (match s with 
+                                 | None -> "<none>"
+                                 | Some s -> s);
+                      pcdata (match i with 
+                                | None -> "<none>"
+                                | Some i -> string_of_int i)];
+                     ])))
+
+(*******)
+let my_nl_params = 
+  Eliom_parameters.make_non_localized_parameters
+    ~prefix:"tutoeliom"
+    ~name:"mynlp"
+    (Eliom_parameters.int "a" ** Eliom_parameters.string "s")
+
+let void_with_nlp =
+  Eliom_services.add_non_localized_get_parameters
+    my_nl_params Eliom_services.void_hidden_coservice'
+
+let nlparams = new_service
+    ~path:["voidnl"]
+    ~get_params:(suffix_prod (int "year" ** int "month") (int "w" ))
+    ()
+
+let nlparams_with_nlp =
+  Eliom_services.add_non_localized_get_parameters
+    my_nl_params nlparams
+
+let () = register
+  nlparams
+  (fun sp ((aa, bb), w) () ->
+     Lwt.return
+       (html
+          (head (title (pcdata "")) [])
+          (body [p [
+                   a void_with_nlp
+                     sp [pcdata "void coservice with non loc param"] ((), (11, "aa"));
+                   a nlparams_with_nlp
+                     sp [pcdata "myself with non loc param"] (((4, 5), 777), (12, "ab"))];
+                 p [pcdata "I have my suffix, ";
+                    pcdata ("with values year = "^string_of_int aa^
+                              " and month = "^string_of_int bb^
+                              ". w = "^string_of_int w^".")];
+                 (match Eliom_parameters.get_non_localized_get_parameters
+                    sp my_nl_params 
+                  with
+                    | None -> 
+                        p [pcdata "I do not have my non localized parameters"]
+                    | Some (a, s) -> 
+                        p [pcdata "I have my non localized parameters, ";
+                           pcdata ("with values a = "^string_of_int a^
+                                     " and s = "^s^".")]
+                 )]))
+    )
+
+
+
+
+(*******)
+(* doing requests *)
+let extreq = 
+  register_new_service
+    ~path:["extreq"] 
+    ~get_params:unit
+    (fun sp () () ->
+       Ocsigen_http_client.get "ocsigen.org" "/" () >>= fun frame ->
+       (match frame.Ocsigen_http_frame.frame_content with
+         | None -> Lwt.return ""
+         | Some stream -> Ocsigen_stream.string_of_stream (Ocsigen_stream.get stream)) >>= fun s ->
+       (* Here use an XML parser, 
+          or send the stream directly using an appropriate Eliom_mkreg module *)
+       return
+         (html
+            (head (title (pcdata "")) [])
+            (body [p [pcdata s]])))
+
+let servreq = 
+  register_new_service
+    ~path:["servreq"] 
+    ~get_params:unit
+    (fun sp () () ->
+       let ri = Eliom_sessions.get_ri sp in
+       let ri = Ocsigen_extensions.ri_of_url "tuto/" ri in
+       Ocsigen_extensions.serve_request ri >>= fun result ->
+       let stream = fst result.Ocsigen_http_frame.res_stream in
+       Ocsigen_stream.string_of_stream (Ocsigen_stream.get stream) >>= fun s ->
+       (* Here use an XML parser, 
+          or send the stream directly using an appropriate Eliom_mkreg module *)
+       return
+         (html
+            (head (title (pcdata "")) [])
+            (body [p [pcdata s]])))
+
+let servreqloop = 
+  register_new_service
+    ~path:["servreqloop"] 
+    ~get_params:unit
+    (fun sp () () ->
+       let ri = Eliom_sessions.get_ri sp in
+       Ocsigen_extensions.serve_request ri >>= fun result ->
+       let stream = fst result.Ocsigen_http_frame.res_stream in
+       Ocsigen_stream.string_of_stream (Ocsigen_stream.get stream) >>= fun s ->
+       (* Here use an XML parser, 
+          or send the stream directly using an appropriate Eliom_mkreg module *)
+       return
+         (html
+            (head (title (pcdata "")) [])
+            (body [p [pcdata s]])))
+
+
+
+
+
+(* Customizing HTTP headers *)
+let headers = 
+  register_new_service
+    ~code:666
+    ~charset:"plopcharset"
+(*    ~content_type:"custom/contenttype" *)
+    ~cookies:[Eliom_services.Set (Some [], None,
+                                  "Customcookie", 
+                                  "Value",
+                                  true);
+              Eliom_services.Set (Some [], None,
+                                  "Customcookie2", 
+                                  "Value2",
+                                  true);
+             ]
+    ~headers:(Http_headers.add
+                (Http_headers.name "XCustom-header")
+                "This is an example" 
+                Http_headers.empty)
+    ~path:["httpheaders"] 
+    ~get_params:unit
+    (fun sp () () ->
+       return
+         (html
+            (head (title (pcdata "")) [])
+            (body [h1 [pcdata "Look at my HTTP headers"]])))
+
+
+(* form towards a suffix service with constants *)
+let create_form (n1, (_, n2)) =
+    <:xmllist< <p>
+      $string_input ~input_type:`Text ~name:n1 ()$
+      $string_input ~input_type:`Text ~name:n2 ()$
+      $string_input ~input_type:`Submit ~value:"Click" ()$</p> >>
+
+let constform = register_new_service ["constform"] unit
+  (fun sp () () ->
+     let f = get_form Tutoeliom.constfix sp create_form in
+     return
+        (html
+          (head (title (pcdata "")) [])
+          (body [h1 [pcdata "Hallo"];
+                 f ])))
+
+
 (* Suffix and other service at same URL *)
 let su2 =
   register_new_service
     ~path:["fuffix";""]
-    ~get_params:(suffix (all_suffix "s"))
+    ~get_params:(suffix (all_suffix_string "s"))
     (fun _ s () ->
       return
         (html
           (head (title (pcdata "")) [])
           (body [h1
-                   [pcdata 
-                      (Ocsigen_extensions.string_of_url_path
-                         ~encode:false s)];
-                 p [pcdata "Try page fuffix/a"]])))
+                   [pcdata s];
+                 p [pcdata "Try page fuffix/a/b"]])))
 
 let su =
   register_new_service
@@ -55,6 +410,29 @@ let su =
           (head (title (pcdata "")) [])
           (body [h1 [pcdata "Try another suffix"]])))
 
+let su3 =
+  register_new_service
+    ~path:["fuffix";""]
+    ~get_params:unit
+    (fun _ () () ->
+      return
+        (html
+          (head (title (pcdata "")) [])
+          (body [h1 [pcdata "Try another suffix"]])))
+
+let create_suffixform_su2 s =
+    <:xmllist< <p>Write a string:
+      $string_input ~input_type:`Text ~name:s ()$ <br/>
+      $string_input ~input_type:`Submit ~value:"Click" ()$</p> >>
+
+let suffixform_su2 = register_new_service ["suffixform_su2"] unit
+  (fun sp () () ->
+     let f = get_form su2 sp create_suffixform_su2 in
+     return
+       (html
+          (head (title (pcdata "")) [])
+          (body [h1 [pcdata "Hallo"];
+                 f ])))
 
 (* optional parameters *)
 let optparam =
@@ -144,7 +522,7 @@ let preappliedsuffix =
 
 let url_encoding =
   register_new_service
-    ~path:["urlencoding"]
+    ~path:["urlencoding&/=?ablah"]
     ~get_params:(suffix_prod (all_suffix "s//\\") any)
     (fun sp (suf, l) () ->
       let ll =
@@ -297,15 +675,15 @@ let getact =
     ~get_params:(int "p")
     ()
 
-let act = Actions.register_new_coservice
+let act = Action.register_new_coservice
     ~fallback:(preapply getact 22)
     ~get_params:(int "bip")
-    (fun _ g p -> v := g; return [])
+    (fun _ g p -> v := g; return ())
 
 (* action on GET non-attached coservice on GET coservice page *)
-let naact = Actions.register_new_coservice'
+let naact = Action.register_new_coservice'
     ~get_params:(int "bop")
-    (fun _ g p -> v := g; return [])
+    (fun _ g p -> v := g; return ())
 
 let naunit = Unit.register_new_coservice'
     ~get_params:(int "bap")
@@ -347,7 +725,7 @@ let _ = Cookies.register cookies
       ((html
         (head (title (pcdata "")) [])
         (body [p
-                 (Ocsigen_http_frame.Cookievalues.fold
+                 (Ocsigen_lib.String_Table.fold
                     (fun n v l ->
                       (pcdata (n^"="^v))::
                       (br ())::l
@@ -387,7 +765,7 @@ let _ = Cookies.register cookies
                               (string_of_int (Random.int 100)),
                               true);
         ]
-       in if Ocsigen_http_frame.Cookievalues.mem (cookiename^"1") (get_cookies sp)
+       in if Ocsigen_lib.String_Table.mem (cookiename^"1") (get_cookies sp)
        then
          (Eliom_services.Unset (None, (cookiename^"1")))::
          (Eliom_services.Unset (None, (cookiename^"2")))::cookies
@@ -411,7 +789,11 @@ let sendany =
      if s = "nocookie"
      then
        Xhtml.send
-         sp
+         ~headers:(Http_headers.add
+                     (Http_headers.name "XCustom-header")
+                     "This is an example" 
+                     Http_headers.empty)
+         ~sp
          (html
             (head (title (pcdata "")) [])
             (body [p [pcdata "This page does not set cookies"]]))
@@ -442,7 +824,7 @@ let sendfile2 =
     ~path:["files";""]
     ~get_params:(suffix (all_suffix "filename"))
     (fun _ s () ->
-      return ("/var/www/ocsigen/"^(Ocsigen_extensions.string_of_url_path ~encode:false s)))
+      return ("/var/www/ocsigen/"^(Ocsigen_lib.string_of_url_path ~encode:false s)))
 
 let sendfileexception =
   register_new_service
@@ -457,22 +839,30 @@ let sendfileexception =
 
 (* Complex suffixes *)
 let suffix2 =
-  register_new_service
+  new_service
     ~path:["suffix2";""]
     ~get_params:(suffix (string "suff1" ** int "ii" ** all_suffix "ee"))
-    (fun sp (suf1,(ii,ee)) () ->
+    ()
+
+let _ =
+  register suffix2
+    (fun sp (suf1, (ii, ee)) () ->
       return
         (html
            (head (title (pcdata "")) [])
            (body
               [p [pcdata "The suffix of the url is ";
                   strong [pcdata (suf1^", "^(string_of_int ii)^", "^
-                                  (Ocsigen_extensions.string_of_url_path ~encode:false ee))]]])))
+                                  (Ocsigen_lib.string_of_url_path ~encode:false ee))]];
+              p [a suffix2 sp [pcdata "link to myself"] ("a", (2, []))]])))
 
 let suffix3 =
   register_new_service
     ~path:["suffix3";""]
-    ~get_params:(suffix_prod (string "suff1" ** int "ii" ** all_suffix_user int_of_string string_of_int "ee") (string "a" ** int "b"))
+    ~get_params:(suffix_prod
+                   (string "suff1" ** int "ii" ** 
+                      all_suffix_user int_of_string string_of_int "ee")
+                   (string "a" ** int "b"))
     (fun sp ((suf1, (ii, ee)), (a, b)) () ->
       return
         (html
@@ -483,12 +873,13 @@ let suffix3 =
                                   (string_of_int ee)^", "^
                                   a^", "^(string_of_int b))]]])))
 
-let create_suffixform2 (suf1,(ii,ee)) =
+let create_suffixform2 (suf1, (ii, ee)) =
     <:xmllist< <p>Write a string:
       $string_input ~input_type:`Text ~name:suf1 ()$ <br/>
       Write an int: $int_input ~input_type:`Text ~name:ii ()$ <br/>
-      Write a string: $user_type_input ~input_type:`Text ~name:ee
-                         (Ocsigen_extensions.string_of_url_path ~encode:false)$ <br/>
+      Write a string: $user_type_input
+      (Ocsigen_lib.string_of_url_path ~encode:false)
+      ~input_type:`Text ~name:ee ()$ <br/>
       $string_input ~input_type:`Submit ~value:"Click" ()$</p> >>
 
 let suffixform2 = register_new_service ["suffixform2"] unit
@@ -520,7 +911,7 @@ let suffixform3 = register_new_service ["suffixform3"] unit
 
 let suffix5 =
   register_new_service
-    ~path:["suffix5";""]
+    ~path:["suffix5"]
     ~get_params:(suffix (all_suffix "s"))
     (fun sp s () ->
       return
@@ -528,7 +919,8 @@ let suffix5 =
            (head (title (pcdata "")) [])
            (body
               [p [pcdata "This is a page with suffix ";
-                  strong [pcdata (Ocsigen_lib.string_of_url_path ~encode:false s)]]])))
+                  strong [pcdata (Ocsigen_lib.string_of_url_path
+                                    ~encode:false s)]]])))
 
 let nosuffix =
   register_new_service
@@ -563,9 +955,15 @@ let sendfile2 =
   Files.register_new_service
     ~path:["files2";""]
 (*    ~get_params:(regexp r "/home/$1/public_html$2" "filename") *)
-    ~get_params:(regexp r "$$u($1)$2" "filename")
+    ~get_params:(suffix ~redirect_if_not_suffix:false
+                   (all_suffix_regexp r "$u($1)/public_html$2" 
+                      ~to_string:(fun s -> s) "filename"))
     (fun _ s () -> return s)
 
+(* Here I am using redirect_if_not_suffix:false because 
+   otherwise I would need to write a more sophisticated to_string function *)
+
+(*
 let sendfile2 =
   Files.register_new_service
     ~path:["files2";""]
@@ -573,6 +971,7 @@ let sendfile2 =
                    (all_suffix_regexp r "/home/$1/public_html$2" "filename"))
 (*    ~get_params:(suffix (all_suffix_regexp r "$$u($1)$2" "filename")) *)
     (fun _ s () -> return s)
+*)
 
 let create_suffixform4 n =
     <:xmllist< <p>Write the name of the file:
@@ -676,10 +1075,150 @@ let any5 = register_new_service
        </body>
      </html> >>)
 
-(* list cannot be in suffix: (not checked) *)
-let sufli = register_new_service
+(* list in suffix *)
+let sufli = new_service
     ~path:["sufli"]
-    ~get_params:(suffix (list "l" (string "s")))
+    ~get_params:(suffix (list "l" (string "s" ** int "i")))
+    ()
+
+let _ = register sufli
+  (fun sp l () ->
+    let ll =
+      List.map
+        (fun (s, i) -> << <strong> $str:(s^string_of_int i)$ </strong> >>) l
+    in
+    return
+  << <html>
+       <head><title></title></head>
+       <body>
+       <p>
+         You sent:
+         <span>$list:ll$</span>
+       </p>
+       <p>
+         $a sufli sp [pcdata "myself"] [("a", 2)]$, 
+         $a sufli sp [pcdata "myself (empty list)"] []$
+       </p>
+       </body>
+     </html> >>)
+
+let create_sufliform f =
+  let l =
+    f.it (fun (sn, iname) v init ->
+            (tr (td [pcdata ("Write a string: ")])
+               [td [string_input ~input_type:`Text ~name:sn ()];
+                td [pcdata ("Write an integer: ")];
+                td [int_input ~input_type:`Text ~name:iname ()];
+               ])::init)
+      ["one";"two";"three"]
+      []
+  in
+  [table (List.hd l) (List.tl l);
+   p [string_input ~input_type:`Submit ~value:"Click" ()]]
+
+let sufliform = register_new_service ["sufliform"] unit
+  (fun sp () () ->
+     let f = get_form sufli sp create_sufliform in
+     return
+       (html
+          (head (title (pcdata "")) [])
+          (body [h1 [pcdata "Hallo"];
+                 f ])))
+
+(*
+(* mmmh ... disabled dynamically for now *)
+let sufli2 = new_service
+    ~path:["sufli2"]
+    ~get_params:(suffix ((list "l" (int "i")) ** int "j"))
+    ()
+
+let _ = register sufli2
+  (fun sp (l, j) () ->
+    let ll =
+      List.map (fun i -> << <strong> $str:(string_of_int i)$ </strong> >>) l
+    in
+    return
+  << <html>
+       <head><title></title></head>
+       <body>
+       <p>
+         You sent:
+         <span>$list:ll$</span>,
+         and
+         j=$str:string_of_int j$.
+       </p>
+       <p>
+         $a sufli2 sp [pcdata "myself"] ([1; 2], 3)$, 
+         $a sufli2 sp [pcdata "myself (empty list)"] ([], 1)$
+       </p>
+       </body>
+     </html> >>)
+*)
+
+let sufliopt = new_service
+    ~path:["sufliopt"]
+    ~get_params:(suffix (list "l" (opt (string "s"))))
+    ()
+
+let _ = register sufliopt
+  (fun sp l () ->
+    let ll =
+      List.map
+        (function None -> pcdata "<none>"
+           | Some s -> << <strong> $str:s$ </strong> >>) l
+    in
+    return
+  << <html>
+       <head><title></title></head>
+       <body>
+       <p>
+         You sent:
+         <span>$list:ll$</span>
+       </p>
+       <p>
+         $a sufliopt sp [pcdata "myself"] [Some "a"; None; Some "po"; None; None; Some "k"; None]$, 
+         $a sufliopt sp [pcdata "myself (empty list)"] []$
+         $a sufliopt sp [pcdata "myself (list [None; None])"] [None; None]$
+         $a sufliopt sp [pcdata "myself (list [None])"] [None]$
+       </p>
+       </body>
+     </html> >>)
+
+
+let sufliopt2 = new_service
+    ~path:["sufliopt2"]
+    ~get_params:(suffix (list "l" (opt (string "s" ** string "ss"))))
+    ()
+
+let _ = register sufliopt2
+  (fun sp l () ->
+    let ll =
+      List.map
+        (function None -> pcdata "<none>"
+           | Some (s, ss) -> << <strong> ($str:s$, $str:ss$) </strong> >>) l
+    in
+    return
+  << <html>
+       <head><title></title></head>
+       <body>
+       <p>
+         You sent:
+         <span>$list:ll$</span>
+       </p>
+       <p>
+         $a sufliopt2 sp [pcdata "myself"] [Some ("a", "jj"); None; Some ("po", "jjj"); None; None; Some ("k", "pp"); None]$, 
+         $a sufliopt2 sp [pcdata "myself (empty list)"] []$
+         $a sufliopt2 sp [pcdata "myself (list [None; None])"] [None; None]$
+         $a sufliopt2 sp [pcdata "myself (list [None])"] [None]$
+       </p>
+       </body>
+     </html> >>)
+
+
+(* set in suffix *)
+let sufset = register_new_service
+    ~path:["sufset"]
+    ~get_params:(suffix (set string "s"))
   (fun _ l () ->
     let ll =
       List.map
@@ -697,6 +1236,7 @@ let sufli = register_new_service
      </html> >>)
 
 
+
 (* form to any2 *)
 let any2form = register_new_service
     ~path:["any2form"]
@@ -741,9 +1281,9 @@ let create_listform f =
      end of the list created
    *)
   let l =
-    f.it (fun boolname v ->
-      [tr (td [pcdata ("Write the value for "^v^": ")])
-         [td [bool_checkbox ~name:boolname ()]]])
+    f.it (fun boolname v init ->
+            (tr (td [pcdata ("Write the value for "^v^": ")])
+               [td [bool_checkbox ~name:boolname ()]])::init)
       ["one";"two";"three"]
       []
   in
@@ -841,7 +1381,7 @@ let uploadgetform = register_new_service ["uploadget"] unit
 
 (*******)
 (* Actions that raises an exception *)
-let exn_act = Actions.register_new_coservice'
+let exn_act = Action.register_new_coservice'
     ~get_params:unit
     (fun _ g p -> fail Not_found)
 
@@ -878,11 +1418,15 @@ let close_from_outside =
 let set_timeout =
 register_new_service
     ~path:["set_timeout"]
-    ~get_params:(int "t" ** bool "recompute")
-    (fun sp (t, recompute) () ->
-      set_global_persistent_data_session_timeout ~session_name:"persistent_sessions"
+    ~get_params:(int "t" ** (bool "recompute" ** bool "overrideconfig"))
+    (fun sp (t, (recompute, override_configfile)) () ->
+      set_global_persistent_data_session_timeout
+        ~override_configfile
+        ~session_name:(Some "persistent_sessions")
         ~recompute_expdates:recompute ~sp (Some (float_of_int t));
-      set_global_volatile_session_timeout ~session_name:"action_example2"
+      set_global_volatile_session_timeout
+        ~override_configfile
+        ~session_name:(Some "action_example2")
         ~recompute_expdates:recompute ~sp (Some (float_of_int t));
       return
         (html
@@ -896,12 +1440,15 @@ register_new_service
 
 
 let create_form =
-  (fun (number_name, boolname) ->
+  (fun (number_name, (bool1name, bool2name)) ->
     [p [pcdata "New timeout: ";
         Eliom_predefmod.Xhtml.int_input ~input_type:`Text ~name:number_name ();
         br ();
         pcdata "Check the box if you want to recompute all timeouts: ";
-        Eliom_predefmod.Xhtml.bool_checkbox ~name:boolname ();
+        Eliom_predefmod.Xhtml.bool_checkbox ~name:bool1name ();
+        br ();
+        pcdata "Check the box if you want to override configuration file: ";
+        Eliom_predefmod.Xhtml.bool_checkbox ~name:bool2name ();
         Eliom_predefmod.Xhtml.string_input ~input_type:`Submit ~value:"Submit" ()]])
 
 let set_timeout_form =
@@ -951,10 +1498,13 @@ let mainpage = register_new_service ["tests"] unit
          a getcoex sp [pcdata "GET coservice with preapplied fallback, etc"] (); br ();
          a postcoex sp [pcdata "POST service with coservice fallback"] (); br ();
          a su sp [pcdata "Suffix and other service at same URL"] (); br ();
+         a suffixform_su2 sp [pcdata "Suffix and other service at same URL: a form towards the suffix service"] (); br ();
          a preappliedsuffix sp [pcdata "Preapplied suffix"] (); br ();
+         a constform sp [pcdata "Form towards suffix service with constants"] (); br ();
          a getact sp [pcdata "action on GET attached coservice, etc"] 127; br ();
          a cookies sp [pcdata "Many cookies"] "le suffixe de l'URL"; br ();
          a sendany sp [pcdata "Cookie or not with Any"] "change this suffix to \"nocookie\""; br ();
+         a headers sp [pcdata "Customizing HTTP headers"] (); br ();
          a sendfileex sp [pcdata "Send file"] (); br ();
          a sendfile2 sp [pcdata "Send file 2"] "style.css"; br ();
          a sendfileexception sp [pcdata "Do not send file"] (); br ();
@@ -972,7 +1522,12 @@ let mainpage = register_new_service ["tests"] unit
          a any5 sp [pcdata "Suffix + any parameters"]
            ("ee", [("bo","ba");("bi","bu")]); br ();
          a uploadgetform sp [pcdata "Upload with GET"] (); br ();
-(* broken        a sufli sp [pcdata "List in suffix"] ["bo";"ba";"bi";"bu"]; br ();*)
+         a sufli sp [pcdata "List in suffix"] [("bo", 4);("ba", 3);("bi", 2);("bu", 1)]; br ();
+         a sufliform sp [pcdata "Form to list in suffix"] (); br ();
+         a sufliopt sp [pcdata "List of optional values in suffix"] [None; Some "j"]; br ();
+         a sufliopt2 sp [pcdata "List of optional pairs in suffix"] [None; Some ("j", "ee")]; br ();
+         a sufset sp [pcdata "Set in suffix"] ["bo";"ba";"bi";"bu"]; br ();
+(*         a sufli2 sp [pcdata "List not in the end of in suffix"] ([1; 2; 3], 4); br (); *)
          a boollistform sp [pcdata "Bool list"] (); br ();
          a preappmenu sp [pcdata "Menu with pre-applied services"] (); br ();
          a exn_act_main sp [pcdata "Actions that raises an exception"] (); br ();
@@ -982,14 +1537,31 @@ let mainpage = register_new_service ["tests"] unit
            ~fragment:"a--   ---++&/@"
            ~service:url_encoding ~sp
            [pcdata "Urls with strange characters inside"]
-           (["l/l%l      &l=l+l)l@";"m\\m\"m";"nn~n"],
-            [("po?po&po~po/po+po", "lo\"l     o#lo'lo lo=lo&l      o/lo+lo");
-            ("bo=mo@co:ro", "zo^zo%zo$zo:zo")]); br ();
-         a ~service:(static_dir_with_params ~sp ~get_params:Eliom_parameters.any)
+           (["l/l%l      &l=l+l)l@";"m\\m\"m";"n?n~n"],
+            [("po?po&po~po/po+po", "lo?\"l     o#lo'lo lo=lo&l      o/lo+lo");
+            ("bo=mo@co:ro", "zo^zo%zo$zo:zo?aaa")]); br ();
+         a ~service:(static_dir_with_params ~sp ~get_params:Eliom_parameters.any ())
            ~sp 
            [pcdata "Static file with GET parameters"]
            (["ocsigen5.png"], [("aa", "lmk"); ("bb", "4")]); br ();
 
+         a extreq sp [pcdata "External request"] (); br ();
+         a servreq sp [pcdata "Server request"] (); br ();
+         a servreqloop sp [pcdata "Looping server request"] (); br ();
+
+         a nlparams sp [pcdata "nl params and suffix, on void coservice"] ((3, 5), 222); br ();
+         a optsuf sp [pcdata "optional suffix"] None; br ();
+         a optsuf sp [pcdata "optional suffix"] (Some ("<a to/to=3?4=2>", None)); br ();
+         a optsuf sp [pcdata "optional suffix"] (Some ("toto", Some 2)); br ();
+         a optsuf2 sp [pcdata "optional suffix 2"] (Some "un", Some 2); br ();
+         a optsuf2 sp [pcdata "optional suffix 2"] (None, Some 2); br ();
+         a optsuf2 sp [pcdata "optional suffix 2"] (Some "un", None); br ();
+         a optsuf2 sp [pcdata "optional suffix 2"] (None, None); br ();
+
+         a csrfsafe_get_example sp [pcdata "GET CSRF safe service"] (); br ();
+         a csrfsafe_postget_example sp [pcdata "POST CSRF safe service on GET CSRF safe service"] (); br ();
+         a csrfsafe_session_example sp [pcdata "POST non attached CSRF safe service in session table"] (); br ();
+         a unregister_example sp [pcdata "Unregistering services"] (); br ();
 
 
        ]])))
