@@ -3,7 +3,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation, with linking exception; 
+ * the Free Software Foundation, with linking exception;
  * either version 2.1 of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -18,18 +18,18 @@
 
 (* A page providing infos about the server (number of sessions, uptime...) *)
 
-open Extensions (* for profiling info *)
-open Eliompredefmod.Xhtml
-open Eliompredefmod
-open Eliomservices
-open Eliomparameters
-open Eliomsessions
+open Ocsigen_extensions (* for profiling info *)
+open Eliom_predefmod.Xhtml
+open Eliom_predefmod
+open Eliom_services
+open Eliom_parameters
+open Eliom_sessions
 open Unix
 open Lwt
 
 let launchtime = Unix.time ()
 
-let _ = 
+let _ =
   register_new_service
     ~path:[]
     ~get_params:unit
@@ -51,43 +51,44 @@ let _ =
       let majcol = string_of_int stat.Gc.major_collections in
       let top_heap_words = string_of_int stat.Gc.top_heap_words in
       let pid = string_of_int (Unix.getpid ()) in
-      let fd = 
+      let fd =
         try
-          let dir = Unix.opendir 
+          let dir = Unix.opendir
               ("/proc/"^pid^"/fd") in
           let rec aux v =
-            try ignore ((* print_endline *) (readdir dir)); aux (v+1) 
+            try ignore ((* print_endline *) (readdir dir)); aux (v+1)
             with End_of_file -> v
           in
-          let r = 
-            try string_of_int ((aux 0) - 2) 
+          let r =
+            try string_of_int ((aux 0) - 2)
             with e -> ("(Error: "^(Printexc.to_string e)^")") in
           Unix.closedir dir;
           Some r
         with _ -> None
       in
-      let nssess = Eliomsessions.number_of_service_sessions sp in
-      let ndsess = Eliomsessions.number_of_volatile_data_sessions sp in
-      let ntables = Eliomsessions.number_of_tables () in
-      let ntableselts = Eliomsessions.number_of_table_elements () in
-      Eliomsessions.number_of_persistent_data_sessions () >>=
+      let nssess = Eliom_sessions.number_of_service_sessions sp in
+      let ndsess = Eliom_sessions.number_of_volatile_data_sessions sp in
+      let ntables = Eliom_sessions.number_of_tables () in
+      let ntableselts = Eliom_sessions.number_of_table_elements () in
+      Eliom_sessions.number_of_persistent_data_sessions () >>=
         (fun nbperssess ->
-          let nbperstab = Eliomsessions.number_of_persistent_tables () in
-          Eliomsessions.number_of_persistent_table_elements () >>=
+          let nbperstab = Eliom_sessions.number_of_persistent_tables () in
+          Eliom_sessions.number_of_persistent_table_elements () >>=
           (fun nbperstabel ->
             let dot = <:xmllist< . >> in
             let list1 a l = <:xmllist<  with, respectively
              $str:List.fold_left
-             (fun deb i -> deb^", "^(string_of_int i)) 
-             (string_of_int a) l $ 
+             (fun deb i -> deb^", "^(string_of_int i))
+             (string_of_int a) l $
            elements inside. >>
            in
             let list2 (n,a) l = <:xmllist<  with, respectively
              $str:List.fold_left
-             (fun deb (s, i) -> deb^", "^s^" : "^(string_of_int i)) 
-             (n^" : "^(string_of_int a)) l$ 
+             (fun deb (s, i) -> deb^", "^s^" : "^(string_of_int i))
+             (n^" : "^(string_of_int a)) l$
            elements inside. >>
             in
+            Eliommod_sessiongroups.Pers.length () >>= fun persgrplength ->
 Lwt.return
 <<
  <html>
@@ -95,10 +96,10 @@ Lwt.return
    </head>
    <body>
      <h1>Ocsigen server monitoring</h1>
-      <p>Version of Ocsigen: $str:Ocsiconfig.version_number$</p>
+      <p>Version of Ocsigen: $str:Ocsigen_config.version_number$</p>
      <p>Uptime: $str:uptime$.</p>
-     <p>The number of sessions is not available in this version of ocsigenmod.</p>
-     <p>Number of clients connected: 
+     <p>The number of sessions is not available in this version of Eliom.</p>
+     <p>Number of clients connected:
          $str:(string_of_int (get_number_of_connected ()))$.</p>
      <p>PID : $str:pid$</p>
      <p>$str:match fd with
@@ -114,9 +115,9 @@ Lwt.return
        </ul>
      <h2>Lwt threads</h2>
      <p>
-       $str:(string_of_int (Lwt_unix.inputs_length ()))$ 
+       $str:(string_of_int (Lwt_unix.inputs_length ()))$
              lwt threads waiting for inputs<br/>
-       $str:(string_of_int (Lwt_unix.outputs_length ()))$ 
+       $str:(string_of_int (Lwt_unix.outputs_length ()))$
                lwt threads waiting for outputs<br/>
        $str:(string_of_int (Lwt_unix.wait_children_length ()))$
                lwt threads waiting for children<br/>
@@ -125,18 +126,18 @@ Lwt.return
        $str:(string_of_int (Lwt_unix.get_new_sleeps ()))$ new sleeps.<br/>
      </p>
      <h2>Preemptive threads</h2>
-     <p>There are currently $str:(string_of_int (Preemptive.nbthreads ()))$ 
+     <p>There are currently $str:(string_of_int (Lwt_preemptive.nbthreads ()))$
              detached threads running
-             (min $str:(string_of_int (Ocsiconfig.get_minthreads ()))$,
-             max $str:(string_of_int (Ocsiconfig.get_maxthreads ()))$),
-      from which $str:(string_of_int (Preemptive.nbthreadsbusy ()))$ are busy.
-        $str:(string_of_int (Preemptive.nbthreadsqueued ()))$ computations 
-           queued (max 
-        $str:(string_of_int (Ocsiconfig.get_max_number_of_threads_queued ()))$).</p>
+             (min $str:(string_of_int (Ocsigen_config.get_minthreads ()))$,
+             max $str:(string_of_int (Ocsigen_config.get_maxthreads ()))$),
+      from which $str:(string_of_int (Lwt_preemptive.nbthreadsbusy ()))$ are busy.
+        $str:(string_of_int (Lwt_preemptive.nbthreadsqueued ()))$ computations
+           queued (max
+        $str:(string_of_int (Ocsigen_config.get_max_number_of_threads_queued ()))$).</p>
      <h2>HTTP connexions</h2>
      <p>Number of
         $str:"" (* string_of_int
-                     (Http_com.Timeout.nb_threads_waiting_timeout ()) *)$
+                     (Ocsigen_http_com.Timeout.nb_threads_waiting_timeout ()) *)$
      connexions waiting for timeout: not implemented in that version</p>
      <h2>Eliom sessions</h2>
      <p>There are $str:string_of_int nssess$ Eliom service sessions opened,
@@ -149,12 +150,18 @@ Lwt.return
        </p>
      <p>There are $str:string_of_int nbperssess$
         Eliom persistent sessions opened.<br/>
-     and $str:string_of_int nbperstab$ Eliom 
+     and $str:string_of_int nbperstab$ Eliom
     persistent tables created $list:
       (match nbperstabel with
       | [] -> dot
       | a::l -> list2 a l)
        $</p>
+      <p>Number of session groups:</p>
+      <ul>
+        <li>Service sessions: $str: string_of_int (Eliommod_sessiongroups.Serv.length ())$</li>
+        <li>Volatile data sessions: $str: string_of_int (Eliommod_sessiongroups.Data.length ())$</li>
+        <li>Persistent data sessions: $str: string_of_int persgrplength$</li>
+      </ul>
    </body>
  </html>
 >>)))
