@@ -172,26 +172,6 @@ let compress deflate stream =
     Ocsigen_stream.make
       ~finalize (fun () -> Ocsigen_stream.cont gzip_header new_stream)
 
-(*****************************************************************************)
-(** Function to be called at the beginning of the initialisation phase
-    of the server (actually each time the config file is reloaded) *)
-let start_init () =
-  ()
-
-(** Function to be called at the end of the initialisation phase *)
-let end_init () =
-  ()
-
-
-
-(*****************************************************************************)
-(** A function that will create an error message from the exceptions
-    that may be raised during the initialisation phase, and raise again
-    all other exceptions. That function has type exn -> string. Use the
-   raise function if you don't need any. *)
-let exn_handler = raise
-
-
 
 (*****************************************************************************)
 (** The filter function *)
@@ -266,7 +246,7 @@ let stream_filter contentencoding url deflate choice res =
                    Some ((if deflate then "Ddeflatemod" else "Gdeflatemod")^e)
                | None -> None);
                Ocsigen_http_frame.res_stream =
-               compress deflate res.Ocsigen_http_frame.res_stream;
+               (compress deflate (fst res.Ocsigen_http_frame.res_stream), None);
                Ocsigen_http_frame.res_headers =
                Http_headers.replace
                  Http_headers.content_encoding
@@ -277,27 +257,18 @@ let stream_filter contentencoding url deflate choice res =
 
 let filter choice_list = function
   | Req_not_found (code,_) -> return (Ext_next code)
-  | Req_found (ri, result) -> result () >>= fun res ->
-  match select_encoding (Lazy.force(ri.ri_accept_encoding)) with
-  | Deflate ->
-      stream_filter "deflate" ri.ri_sub_path_string true choice_list res
-  | Gzip ->
-      stream_filter "gzip" ri.ri_sub_path_string false choice_list res
-  | Id | Star -> return (Ext_found (fun () -> return res))
-  | Not_acceptable ->
-      return (Ext_stop_all (res.Ocsigen_http_frame.res_cookies,406))
+  | Req_found ({ request_info = ri }, res) ->
+      match select_encoding (Lazy.force(ri.ri_accept_encoding)) with
+        | Deflate ->
+            stream_filter "deflate" ri.ri_sub_path_string true choice_list res
+        | Gzip ->
+            stream_filter "gzip" ri.ri_sub_path_string false choice_list res
+        | Id | Star -> return (Ext_found (fun () -> return res))
+        | Not_acceptable ->
+            return (Ext_stop_all (res.Ocsigen_http_frame.res_cookies,406))
 
 
 (*****************************************************************************)
-(** Extensions may take some options from the config file.
-    These options are written in xml inside the <extension> tag.
-   For example:
-   <extension module=".../extensiontemplate.cmo">
-     <myoption myattr="hello">
-        ...
-     </myoption>
-   </extension>
- *)
 
 let rec parse_filter = function
  |[] -> []
@@ -340,38 +311,23 @@ let rec parse_global_config = function
   | _ -> raise (Error_in_config_file
                   "Unexpected content inside deflatemod config")
 
-let _ = parse_global_config (Ocsigen_extensions.get_config ())
 
 
 
 (*****************************************************************************)
-(** Extensions may define new tags for configuring each site.
-    These tags are inside <site ...>...</site> in the config file.
 
-   For example:
-   <site dir="">
-     <extensiontemplate module=".../mymodule.cmo" />
-   </site>
+let parse_config = function
+  | Element ("deflate", [("compress",b)], choices) ->
+      let l = (try parse_filter choices
+               with Not_found -> raise (Error_in_config_file
+                                          "Can't parse filter content")) in
+      (match b with
+         |"only" -> filter (Compress_only l)
+         |"allbut" -> filter (All_but l)
+         | _ ->  raise (Error_in_config_file
+                     "Attribute \"compress\" should be \"allbut\" or \"only\""))
+  | Element ("deflate" as s, _, _) -> badconfig "Bad syntax for tag %s" s
 
-   Each extension will set its own configuration options, for example:
-   <site dir="">
-     <extensiontemplate module=".../mymodule.cmo" />
-     <eliom module=".../myeliommodule.cmo" />
-     <static dir="/var/www/plop" />
-   </site>
-
- *)
-
-let parse_config path charset _ _ = function
-     | Element ("deflate", [("compress",b)], choices) ->
-     let l = (try parse_filter choices
-              with Not_found -> raise (Error_in_config_file
-                                "Can't parse filter content")) in
-     (match b with
-       |"only" -> filter (Compress_only l)
-       |"allbut" -> filter (All_but l)
-       | _ ->  raise (Error_in_config_file
-         "Attribute \"compress\" should be \"allbut\" or \"only\""))
   | Element (t, _, _) -> raise (Bad_config_tag_for_extension t)
   | _ ->
       raise (Error_in_config_file "Unexpected data in config file")
@@ -379,18 +335,11 @@ let parse_config path charset _ _ = function
 
 
 
-(*****************************************************************************)
-let site_creator hostpattern = parse_config
-   (* hostpattern has type Ocsigen_extensions.virtual_hosts
-      and represents the name of the virtual host *)
-
 
 (*****************************************************************************)
 (** Registration of the extension *)
-let _ = Ocsigen_extensions.register_extension
-  site_creator
-  Ocsigen_extensions.void_extension
-  start_init
-  end_init
-  exn_handler
-
+let () = Ocsigen_extensions.register_extension
+  ~name:"deflatemod"
+  ~fun_site:(fun _ _ _ _ -> parse_config)
+  ~init_fun:parse_global_config
+  ()
