@@ -67,7 +67,7 @@ type mode = Answer | Query | Nofirstline
 
 type waiter =
   { w_wait : unit Lwt.t;
-    w_waker : unit Lwt.u option;
+    w_waker: unit Lwt.u option;
     mutable w_did_wait : bool }
 
 let create_waiter block =
@@ -479,9 +479,9 @@ NOT IMPLEMENTED
            Some s
     )
   in
-  Lwt.return {Ocsigen_http_frame.header = header;
-              Ocsigen_http_frame.content = la;
-              Ocsigen_http_frame.abort =
+  Lwt.return {Ocsigen_http_frame.frame_header = header;
+              frame_content = la;
+              frame_abort =
               (fun () -> Lwt_ssl.close receiver.fd; Lwt.return ())}
 (*VVV close or shutdown? *)
 
@@ -709,6 +709,12 @@ let write_stream ?(chunked=false) out_ch stream =
 
 module H = Ocsigen_http_frame.Http_header
 
+let set_result_observer, observe_result =
+  let observer = ref (fun _ _ -> Lwt.return ()) in
+  ((fun f -> 
+      let o = !observer in
+      observer := (fun a b -> o a b >>= fun () -> f a b)),
+   (fun a b -> !observer a b))
 
 (** Sends the HTTP frame.
  * The headers are merged with those of the sender, the priority
@@ -811,6 +817,7 @@ let send
                   Ocsigen_messages.debug2 "writing header";
                   let hh = Framepp.string_of_header hd in
                   Ocsigen_messages.debug2 hh;
+                  observe_result hd hh >>= fun () ->
                   Lwt_chan.output_string out_ch hh >>= fun () ->
                    (if reopen <> None then
                      (* If we want to give a possibility to reopen if
@@ -894,7 +901,7 @@ let send
       | None   -> "")
   in
   let mkcookl path t hds =
-    Cookievalues.fold
+    Ocsigen_lib.String_Table.fold
       (fun name c h ->
         let exp, v, secure = match c with
         | Ocsigen_http_frame.OUnset -> (Some 0., "", false)
