@@ -36,7 +36,6 @@ open Lwt
 open Ocsigen_lib
 
 exception Ocsigen_http_error of (Ocsigen_http_frame.cookieset * int)
-exception Ocsigen_Internal_Error of string
 exception Ocsigen_Looping_request
 
 
@@ -53,21 +52,24 @@ exception Error_in_user_config_file of string
 let badconfig fmt = Printf.ksprintf (fun s -> raise (Error_in_config_file s)) fmt
 
 (*****************************************************************************)
-(** type of URL, without parameter *)
-type url_path = string list
-
-let string_of_url_path = Ocsigen_lib.string_of_url_path
-
-
-type file_info = {tmp_filename: string;
-                  filesize: int64;
-                  raw_original_filename: string;
-                  original_basename: string}
-
-
 (* virtual hosts: *)
-type virtual_host_part = Text of string * int | Wildcard
-type virtual_hosts = ((virtual_host_part list) * int option) list
+type virtual_hosts = (string * Netstring_pcre.regexp * int option) list
+
+(* We cannot use generic comparison, as regexpes are abstract values that
+   cannot be compared or hashed. However the string essentially contains
+   the value that is compiled into a regexp, and we compare this instead *)
+
+let hash_virtual_hosts (l : virtual_hosts) =
+  Hashtbl.hash (List.map (fun (s, _, p) -> (s, p)) l)
+
+let rec equal_virtual_hosts (l1 : virtual_hosts) (l2 : virtual_hosts) =
+  match l1, l2 with
+    | [], [] -> true
+    | [], _::_ | _::_, [] -> false
+    | (s1, _, p1) :: q1, (s2, _, p2) :: q2 ->
+        s1 = s2 && p1 = p2 && equal_virtual_hosts q1 q2
+
+(*****************************************************************************)
 
 type client = Ocsigen_http_com.connection
 
@@ -170,6 +172,9 @@ type config_info = {
 
   do_not_serve_404: do_not_serve;
   do_not_serve_403: do_not_serve;
+
+  uploaddir: string option;
+  maxuploadfilesize: int64 option;
 }
 and follow_symlink =
   | DoNotFollowSymlinks (** Never follow a symlink *)
@@ -199,8 +204,8 @@ type request_info =
      ri_port_from_host_field: int option; (** Port in the host field of the request (if any) *)
      ri_get_params: (string * string) list Lazy.t;  (** Association list of get parameters *)
      ri_initial_get_params: (string * string) list Lazy.t;  (** Association list of get parameters, as sent by the browser (must not be modified by extensions) *)
-     ri_post_params: (string * string) list Lwt.t Lazy.t; (** Association list of post parameters *)
-     ri_files: (string * file_info) list Lwt.t Lazy.t; (** Files sent in the request *)
+     ri_post_params: config_info -> (string * string) list Lwt.t; (** Association list of post parameters *)
+     ri_files: config_info -> (string * file_info) list Lwt.t; (** Files sent in the request *)
      ri_remote_inet_addr: Unix.inet_addr; (** IP of the client *)
      ri_remote_ip: string;            (** IP of the client *)
      ri_remote_ip_parsed: ip_address Lazy.t;    (** IP of the client, parsed *)
@@ -208,7 +213,7 @@ type request_info =
      ri_server_port: int;      (** Port of the request (server) *)
      ri_user_agent: string;    (** User_agent of the browser *)
      ri_cookies_string: string option Lazy.t; (** Cookies sent by the browser *)
-     ri_cookies: string Ocsigen_http_frame.Cookievalues.t Lazy.t;  (** Cookies sent by the browser *)
+     ri_cookies: string Ocsigen_lib.String_Table.t Lazy.t;  (** Cookies sent by the browser *)
      ri_ifmodifiedsince: float option;   (** if-modified-since field *)
      ri_ifunmodifiedsince: float option;   (** if-unmodified-since field *)
      ri_ifnonematch: string list option;   (** if-none-match field ( * and weak entity tags not implemented) *)
@@ -224,15 +229,18 @@ type request_info =
      ri_accept_language: (string * float option) list Lazy.t; (** Accept-Language HTTP header. The float is the "quality" value, if any. *)
 
      ri_http_frame: Ocsigen_http_frame.t; (** The full http_frame *)
-     ri_extension_info: exn list; (** Use this to put anything you want,
-                                      for example, information for subsequent
-                                      extensions
-                                   *)
+     mutable ri_request_cache: Polytables.t;
+     (** Use this to put anything you want,
+         for example, information for subsequent
+         extensions
+     *)
      ri_client: client; (** The request connection *)
      ri_range: ((int64 * int64) list * int64 option * ifrange) option Lazy.t; 
      (** Range HTTP header. [None] means all the document. 
          List of intervals + possibly from an index to the end of the document.
      *)
+     mutable ri_nb_tries: int; (** For internal use: 
+                                   used to prevent loops of requests *)
    }
 and request = {
   request_info: request_info;
@@ -359,7 +367,7 @@ let get_config () = !dynlinkconfig
 
 
 (*****************************************************************************)
-let site_match request site_path url =
+let site_match request (site_path : string list) url =
   (* We are sure that there is no / at the end or beginning of site_path *)
   (* and no / at the beginning of url *)
   (* and no // or ../ inside both of them *)
@@ -485,6 +493,14 @@ let rec default_parse_config
           | Req_found (ri, res) ->
               Lwt.return (Ext_found_continue_with' (res, ri), cookies_to_set)
           | Req_not_found (e, oldri) ->
+              let oldri = match charset with
+                | None -> oldri
+                | Some charset ->
+                    { oldri with request_config =
+                        { oldri.request_config with charset_assoc =
+                            Ocsigen_charset_mime.set_default_charset
+                              oldri.request_config.charset_assoc charset } }
+              in
               match site_match oldri path oldri.request_info.ri_full_path with
               | None ->
                   Ocsigen_messages.debug (fun () ->
@@ -738,80 +754,70 @@ let start_initialisation, during_initialisation,
 (********)
 
 
-let host_match host port =
+let host_match ~(virtual_hosts : virtual_hosts) ~host ~port =
   let port_match = function
     | None -> true
     | Some p -> p = port
-  in
-  let rec aux host =
-    let hostlen = String.length host in
-    let rec host_match1 beg =
-      let rec aux1 t len l p0 =
-        try
-          let (p,_) =
-            Netstring_str.search_forward (Netstring_str.regexp t) host p0 in
-          let beg2 = p + len in
-          (host_match1 beg2 l) || (aux1 t len l (p+1))
-        with Not_found -> false
-      in
-      function
-        | [] -> beg = hostlen
-        | [Wildcard] -> true
-        | (Wildcard)::(Wildcard)::l ->
-            host_match1 beg ((Wildcard)::l)
-        | (Wildcard)::(Text (t,len))::l -> aux1 t len l beg
-        | (Text (t,len))::l ->
-            try
-              (t = String.sub host beg len) && (host_match1 (beg+len) l)
-            with Invalid_argument _ -> false
-    in
-    function
-      | [] -> false
-      | (a, p)::l -> ((port_match p) && (host_match1 0 a)) || aux host l
   in match host with
-    | None -> List.exists (fun (_, p) -> port_match p)
+    | None -> List.exists (fun (_, _, p) -> port_match p) virtual_hosts
         (*VVV Warning! For HTTP/1.0, when host is absent,
-           we take the first one, even if it doesn't match!
-        *)
-    | Some host -> aux host
-
-
-let string_of_host h =
-  let aux1 (hh, port) =
-    let p = match port with
-    | None -> ""
-    | Some a -> ":"^(string_of_int a)
-    in
-    let rec aux2 = function
-      | [] -> ""
-      | Wildcard::l -> "*"^(aux2 l)
-      | (Text (t,_))::l -> t^(aux2 l)
-    in (aux2 hh)^p
-  in List.fold_left (fun d hh -> d^(aux1 hh)^" ") "" h
+           we take the first one, even if it doesn't match!  *)
+    | Some host ->
+        let host_match regexp =
+          (Netstring_pcre.string_match regexp host 0 <> None)
+        in
+        let rec aux = function
+          | [] -> false
+          | (_, a, p)::l -> (port_match p && host_match a) || aux l
+        in
+        aux virtual_hosts
 
 
 
-let do_for_site_matching host port ri =
+(* Currently used only for error messages. *)
+let string_of_host (h : virtual_hosts) =
+  let aux1 (host, _, port) =
+    match port with
+      | None -> host
+      | Some p -> host ^ ":" ^ string_of_int p
+  in List.fold_left (fun d arg -> d ^ aux1  arg ^" ") "" h
+
+
+
+let serve_request
+    ?(previous_cookies = Ocsigen_http_frame.Cookies.empty)
+    ?(awake_next_request = false) ri =
+
+  let host = ri.ri_host in
+  let port = ri.ri_server_port in
 
   let conn = client_connection ri.ri_client in
   let awake =
-    let tobeawoken = ref true in
-    (* must be awoken once and only once *)
-    fun () ->
-      if !tobeawoken then begin
-        tobeawoken := false;
-        Ocsigen_http_com.wakeup_next_request conn
-      end
+    if awake_next_request
+    then
+      (let tobeawoken = ref true in
+       (* must be awoken once and only once *)
+       fun () ->
+         if !tobeawoken then begin
+           tobeawoken := false;
+           Ocsigen_http_com.wakeup_next_request conn
+         end)
+    else Ocsigen_lib.id
   in
 
-  let rec do2 nb_retries sites cookies_to_set ri =
+  let rec do2 sites cookies_to_set ri =
+    ri.ri_nb_tries <- ri.ri_nb_tries + 1;
+    if ri.ri_nb_tries > Ocsigen_config.get_maxretries ()
+    then fail Ocsigen_Looping_request
+    else
     let string_of_host_option = function
       | None -> "<no host>:"^(string_of_int port)
       | Some h -> h^":"^(string_of_int port)
     in
     let rec aux_host ri prev_err cookies_to_set = function
       | [] -> fail (Ocsigen_http_error (cookies_to_set, prev_err))
-      | (h, conf_info, host_function)::l when host_match host port h ->
+      | (h, conf_info, host_function)::l when
+          host_match ~virtual_hosts:h ~host ~port ->
           Ocsigen_messages.debug (fun () ->
             "-------- host found! "^
             (string_of_host_option host)^
@@ -850,16 +856,11 @@ let do_for_site_matching host port ri =
               aux_host ri e
                 (Ocsigen_http_frame.add_cookies cook cookies_to_set) l
           | Ext_retry_with (request2, cook) ->
-              if nb_retries < Ocsigen_config.get_maxretries ()
-              then
-                do2
-                  (nb_retries + 1)
-                  (get_hosts ())
-                  (Ocsigen_http_frame.add_cookies cook cookies_to_set)
-                  request2.request_info
-                  (* retry all *)
-              else
-                fail Ocsigen_Looping_request
+              do2
+                (get_hosts ())
+                (Ocsigen_http_frame.add_cookies cook cookies_to_set)
+                request2.request_info
+                (* retry all *)
           | Ext_sub_result sr ->
               assert false
           )
@@ -873,7 +874,7 @@ let do_for_site_matching host port ri =
   in
   Lwt.finalize
     (fun () ->
-      do2 0 (get_hosts ()) Ocsigen_http_frame.Cookies.empty ri
+      do2 (get_hosts ()) previous_cookies ri
     )
     (fun () ->
        awake ();
@@ -886,20 +887,29 @@ let do_for_site_matching host port ri =
 
 
 (* used to modify the url in ri (for example for retrying after rewrite) *)
-let ri_of_url url ri =
+let ri_of_url ?(full_rewrite = false) url ri =
   let (_, host, _, url, url2, path, params, get_params) = parse_url url in
   let host = match host with
     | Some h -> host
     | None -> ri.ri_host
   in
+  let path_string = string_of_url_path ~encode:true path in
+  let original_fullpath, original_fullpath_string =
+    if full_rewrite
+    then (path, path_string)
+    else (ri.ri_original_full_path, ri.ri_original_full_path_string)
+  in
+     (* ri_original_full_path is not changed *)
   {ri with
    ri_url_string = url;
    ri_url = url2;
    ri_host = host;
-   ri_full_path_string = string_of_url_path ~encode:true path;
+   ri_full_path_string = path_string;
    ri_full_path = path;
-     (* ri_original_full_path is not changed *)
+   ri_original_full_path_string = original_fullpath_string;
+   ri_original_full_path = original_fullpath;
    ri_sub_path = path;
+   ri_sub_path_string = path_string;
    ri_get_params_string = params;
    ri_get_params = get_params;
   }
@@ -909,13 +919,35 @@ let ri_of_url url ri =
 (*****************************************************************************)
 (* This is used by server.ml.
    I put that here because I need it to be accessible for profiling. *)
+let sockets = ref []
+let sslsockets = ref []
+
 let get_number_of_connected,
   incr_connected,
-  decr_connected =
+  decr_connected,
+  wait_fewer_connected =
   let connected = ref 0 in
+  let maxr = ref (-1000) in
+  let mvar = Lwt_mvar.create_empty () in
   ((fun () -> !connected),
-   (fun () -> connected := !connected + 1),
-   (fun () -> connected := !connected - 1))
+   (fun n -> connected := !connected + n),
+   (fun () -> 
+      let c = !connected in
+      connected := c - 1;
+      if !connected <= 0 && !sockets = [] && !sslsockets = []
+      then exit 0;
+      if c = !maxr
+      then begin
+        Ocsigen_messages.warning "Number of connections now ok";
+        maxr := -1000;
+        Lwt_mvar.put mvar ()
+      end
+      else Lwt.return ()
+   ),
+   (fun max -> 
+      maxr := max;
+      Lwt_mvar.take mvar)
+  )
 
 
 
@@ -1024,3 +1056,21 @@ let find_redirection regexp full_url dest
       | Some _ -> (* Matching regexp found! *)
           Netstring_pcre.global_replace regexp dest path
 
+
+(******************************************************************)
+(* Extending commands *)
+exception Unknown_command
+
+let register_command_function, get_command_function =
+  let command_function = ref (fun ?prefix _ _ -> raise Unknown_command) in
+  ((fun ?prefix f -> 
+      let prefix' = prefix in
+      let old_command_function = !command_function in
+      command_function := 
+        (fun ?prefix s c -> 
+           try old_command_function ?prefix s c
+           with Unknown_command -> 
+             if prefix = prefix'
+             then f s c
+             else raise Unknown_command)),
+   (fun () -> !command_function))
