@@ -30,8 +30,6 @@ open Lwt
 open Ocsigen_lib
 
 exception Ocsigen_http_error of (Ocsigen_http_frame.cookieset * int)
-exception Ocsigen_Internal_Error of string
-
 
 (** Xml tag not recognized by an extension (usually not a real error) *)
 exception Bad_config_tag_for_extension of string
@@ -49,16 +47,19 @@ val badconfig : ('a, unit, string, 'b) format4 -> 'a
     a sprintf-formatted argument. *)
 
 (*****************************************************************************)
-(** The type of URL paths. [["plop";"plip"]] corresponds to [plop/plip]. *)
-type url_path = string list
 
-val string_of_url_path : encode:bool -> url_path -> string
+(** Type of the result of parsing the field [hostfiler] in the configuration
+    file. Inside the list, the first argument is the host itself
+    (which is a glob-like pattern that can contains [*]), a regexp
+    parsing this pattern, and optionnaly a port.
+*)
+type virtual_hosts = (string * Netstring_pcre.regexp * int option) list
 
+val hash_virtual_hosts : virtual_hosts -> int
+val equal_virtual_hosts : virtual_hosts -> virtual_hosts -> bool
 
-(* virtual hosts: *)
-type virtual_host_part = Text of string * int | Wildcard
-type virtual_hosts = ((virtual_host_part list) * int option) list
-
+val host_match:
+  virtual_hosts:virtual_hosts -> host:string option -> port:int -> bool
 
 (*****************************************************************************)
 
@@ -108,6 +109,8 @@ type config_info = {
   do_not_serve_404: do_not_serve;
   do_not_serve_403: do_not_serve;
 
+  uploaddir: string option;
+  maxuploadfilesize: int64 option;
 }
 and follow_symlink =
   | DoNotFollowSymlinks (** Never follow a symlink *)
@@ -119,22 +122,6 @@ and follow_symlink =
 (*****************************************************)
 
 
-(** The files sent in the request *)
-type file_info = {tmp_filename: string; (** Where the file is stored on the server*)
-                  filesize: int64; (** Size, in bytes *)
-                  raw_original_filename: string;
-                  (** Original file name, as given by the client. *)
-                  original_basename: string (** Original file name *) }
-(** Note that the files are cancelled once the request has been fulfilled *)
-
-(** Only IE is known to make [raw_original_filename] and
-    [original_basename] differ, as it sends the full original path
-    of uploaded files.  In all cases, [original_basename] is the
-    basename of the file. More precisely, it is the part of the
-    filename after the last [/] or [\ ], if any, or ["none"] if one of
-    these characters is the last one. You should probably never use
-    [raw_original_filename]. *)
-
 type client
 (** A value of this type represents the client who did the request. *)
 
@@ -164,16 +151,18 @@ type request_info =
      ri_port_from_host_field: int option; (** Port in the host field of the request (if any) *)
      ri_get_params: (string * string) list Lazy.t;  (** Association list of get parameters *)
      ri_initial_get_params: (string * string) list Lazy.t;  (** Association list of get parameters, as sent by the browser (must not be modified by extensions) *)
-     ri_post_params: (string * string) list Lwt.t Lazy.t; (** Association list of post parameters *)
-     ri_files: (string * file_info) list Lwt.t Lazy.t; (** Files sent in the request *)
+     ri_post_params: config_info -> (string * string) list Lwt.t; (** Association list of post parameters *)
+     ri_files: config_info -> (string * file_info) list Lwt.t; (** Files sent in the request *)
      ri_remote_inet_addr: Unix.inet_addr; (** IP of the client *)
      ri_remote_ip: string;            (** IP of the client *)
-     ri_remote_ip_parsed: ip_address Lazy.t;    (** IP of the client, parsed *)
+     ri_remote_ip_parsed: Ocsigen_lib.ip_address Lazy.t;    
+     (** IP of the client, using int32 or int64 (parsed from ri_remote_ip).
+         To be used for masks. *)
      ri_remote_port: int;      (** Port used by the client *)
      ri_server_port: int;      (** Port of the request (server) *)
      ri_user_agent: string;    (** User_agent of the browser *)
      ri_cookies_string: string option Lazy.t; (** Cookies sent by the browser *)
-     ri_cookies: string Ocsigen_http_frame.Cookievalues.t Lazy.t;  (** Cookies sent by the browser *)
+     ri_cookies: string Ocsigen_lib.String_Table.t Lazy.t;  (** Cookies sent by the browser *)
      ri_ifmodifiedsince: float option;   (** if-modified-since field *)
      ri_ifunmodifiedsince: float option;   (** if-unmodified-since field *)
      ri_ifnonematch: string list option;   (** if-none-match field ( * and weak entity tags not implemented) *)
@@ -190,15 +179,18 @@ type request_info =
 
 
      ri_http_frame: Ocsigen_http_frame.t; (** The full http_frame *)
-     ri_extension_info: exn list; (** Use this to put anything you want,
-                                      for example, information for subsequent
-                                      extensions
-                                   *)
+     mutable ri_request_cache: Polytables.t;
+     (** Use this to put anything you want,
+         for example, information for subsequent
+         extensions
+     *)
      ri_client: client; (** The request connection *)
      ri_range: ((int64 * int64) list * int64 option * ifrange) option Lazy.t; 
      (** Range HTTP header. [None] means all the document. 
          List of intervals + possibly from an index to the end of the document.
      *)
+     mutable ri_nb_tries: int; (** For internal use: 
+                                   used to prevent loops of requests *)
    }
 (** If you force [ri_files] or [ri_post_params], the request is fully read,
    so it is not possible any more to read it from [ri_http_frame]
@@ -221,7 +213,7 @@ type answer =
           Here is the function to generate the page".
           The extension must return Ext_found as soon as possible
           when it is sure it is safe to start next request.
-          Usually as soon as you know tha the result will be Ext_found.
+          Usually as soon as you know that the result will be Ext_found.
           But in some case, for example proxies, you don't want the request of
           one connection to be handled in different order.
           In that case, wait to be sure that the new request will not
@@ -425,7 +417,7 @@ val get_port : request -> int
    This allows to modify the URL in the request_info.
    (to be used for example with Ext_retry_with or Ext_continue_with)
  *)
-val ri_of_url : string -> request_info -> request_info
+val ri_of_url : ?full_rewrite:bool -> string -> request_info -> request_info
 
 
 (** {3 User directories} *)
@@ -454,6 +446,28 @@ val find_redirection :
   bool ->
   string option -> int -> string option -> string -> string -> string
 
+
+(** {3 Extending server commands} *)
+exception Unknown_command
+
+(** Use a prefix for all your cammands when you want to create
+    extension specific commands. 
+    For example if the prefix is "myextension" and the commande "blah",
+    the actual command to be written by the user is "myextension:blah".
+    Give as parameter the function that will parse the command and do an action.
+    Its first parameter is the full comand as a string.
+    The second one is the command without prefix, split by word.
+    It must raise [ocsigen_extensions.Unknown_command] if it does
+    not recognize the command.
+*)
+val register_command_function :
+  ?prefix:string -> (string -> string list -> unit) -> unit
+
+(**/**)
+val get_command_function :
+  unit -> (?prefix:string -> string -> string list -> unit)
+
+(**/**)
 (**/**)
 
 val make_parse_config : url_path -> parse_config_aux -> parse_fun
@@ -465,10 +479,10 @@ val set_hosts : (virtual_hosts * config_info * extension2) list -> unit
 
 val get_hosts : unit -> (virtual_hosts * config_info * extension2) list
 
-val do_for_site_matching :
-    string option ->
-    int ->
-    request_info -> Ocsigen_http_frame.result Lwt.t
+val serve_request :
+  ?previous_cookies:Ocsigen_http_frame.cookieset ->
+  ?awake_next_request:bool ->
+  request_info -> Ocsigen_http_frame.result Lwt.t
 
 (** Profiling *)
 val get_number_of_connected : unit -> int
@@ -476,8 +490,9 @@ val get_number_of_connected : unit -> int
 
 
 (** Server internal functions: *)
-val incr_connected : unit -> unit
-val decr_connected : unit -> unit
+val incr_connected : int -> unit
+val decr_connected : unit -> unit Lwt.t
+val wait_fewer_connected : int -> unit Lwt.t
 
 val during_initialisation : unit -> bool
 val start_initialisation : unit -> unit
@@ -490,3 +505,6 @@ val set_config : Simplexmlparser.xml list -> unit
 
 val client_of_connection : Ocsigen_http_com.connection -> client
 
+val sockets : Lwt_unix.file_descr list ref
+val sslsockets : Lwt_unix.file_descr list ref
+
