@@ -36,6 +36,9 @@ open Lazy
 exception Ocsigen_unsupported_media
 exception Ssl_Exception
 exception Ocsigen_upload_forbidden
+exception Socket_closed
+
+let shutdown = ref false
 
 let () = Random.self_init ()
 
@@ -52,11 +55,41 @@ let _ =
 external disable_nagle : Unix.file_descr -> unit = "disable_nagle"
 external initgroups : string -> int -> unit = "initgroups_stub"
 
-let option_get_default x d = match x with Some x -> x | None -> d
-let local_addr addr num = 
-  Unix.ADDR_INET (option_get_default addr Unix.inet_addr_any, num)
-let local_addr6 addr num = 
-  Unix.ADDR_INET (option_get_default addr Unix.inet6_addr_any, num)
+let make_ipv6_socket addr port =
+  let socket = Lwt_unix.socket Unix.PF_INET6 Unix.SOCK_STREAM 0 in
+  Lwt_unix.set_close_on_exec socket;
+  Lwt_unix.setsockopt socket Unix.SO_REUSEADDR true;
+  Lwt_unix.setsockopt socket Unix.IPV6_ONLY true;
+  Lwt_unix.bind socket (Unix.ADDR_INET (addr, port));
+  socket
+
+let make_ipv4_socket addr port =
+  let socket = Lwt_unix.socket Unix.PF_INET Unix.SOCK_STREAM 0 in
+  Lwt_unix.set_close_on_exec socket;
+  Lwt_unix.setsockopt socket Unix.SO_REUSEADDR true;
+  Lwt_unix.bind socket (Unix.ADDR_INET (addr, port));
+  socket
+
+let make_sockets addr port =
+  match addr with
+    | All ->
+        (* The user didn't specify a protocol in the configuration
+           file; we try to open an IPv6 socket (listening to IPv6
+           only) if possible and we open an IPv4 socket anyway. This
+           corresponds to the net.ipv6.bindv6only=0 behaviour on Linux,
+           but is portable and should work with
+           net.ipv6.bindv6only=1 as well. *)
+        let ipv6_socket =
+          try [make_ipv6_socket Unix.inet6_addr_any port]
+          with Unix.Unix_error
+              ((Unix.EAFNOSUPPORT | Unix.EPROTONOSUPPORT),
+               _, _) -> []
+        in
+        (make_ipv4_socket Unix.inet_addr_any port)::ipv6_socket
+    | IPv4 addr ->
+        [make_ipv4_socket addr port]
+    | IPv6 addr ->
+        [make_ipv6_socket addr port]
 
 let sslctx = Ocsigen_http_client.sslcontext
 
@@ -71,11 +104,6 @@ let port_of_sockaddr = function
 
 
 let get_boundary ctparams = List.assoc "boundary" ctparams
-(*
-  let (_, res) = Netstring_pcre.search_forward
-      (Netstring_pcre.regexp "boundary=([^;]*);?") cont_enc 0 in
-  Netstring_pcre.matched_group res 1 cont_enc
-*)
 
 let find_field field content_disp =
   let (_, res) = Netstring_pcre.search_forward
@@ -84,7 +112,7 @@ let find_field field content_disp =
 
 type to_write =
     No_File of string * Buffer.t
-  | A_File of (string * string * string * Unix.file_descr)
+  | A_File of (string * string * string * Unix.file_descr * (string * string option) option)
 
 let counter = let c = ref (Random.int 1000000) in fun () -> c := !c + 1 ; !c
 
@@ -99,8 +127,10 @@ let dbg sockaddr s =
        "While talking to " ^ ip ^ ": " ^ s)
 
 
-let rec find_post_params http_frame ct filenames =
-  match http_frame.Ocsigen_http_frame.content with
+let r_content_type = Netstring_pcre.regexp "([^ ]*)"
+
+let rec find_post_params http_frame ct filenames uploaddir =
+  match http_frame.Ocsigen_http_frame.frame_content with
     | None -> return ([], [])
     | Some body_gen ->
         try
@@ -116,6 +146,7 @@ let rec find_post_params http_frame ct filenames =
                 find_post_params_form_urlencoded body_gen
             | "multipart", "form-data" ->
                 find_post_params_multipart_form_data body_gen ctparams filenames
+                  uploaddir
             | _ -> fail Ocsigen_unsupported_media
         with e -> Lwt.fail e
 
@@ -134,18 +165,31 @@ and find_post_params_form_urlencoded body_gen =
        | Ocsigen_stream.String_too_large -> fail Input_is_too_large
        | e -> fail e)
 
-and find_post_params_multipart_form_data body_gen ctparams filenames =
+and find_post_params_multipart_form_data body_gen ctparams filenames ci =
   (* Same question here, should this stream be consumed after an error ? *)
   let body = Ocsigen_stream.get body_gen
   and bound = get_boundary ctparams
   and params = ref []
   and files = ref [] in
   let create hs =
+    let content_type =
+      try
+        let ct = List.assoc "content-type" hs in
+        let content_type =
+          let (_, res) = Netstring_pcre.search_forward r_content_type ct 0 in
+          Netstring_pcre.matched_group res 1 ct
+        in
+        let charset = try
+          Some (find_field "charset" ct)
+        with _ -> None
+        in Some (content_type, charset)
+      with _ -> None
+    in
     let cd = List.assoc "content-disposition" hs in
     let p_name = find_field "name" cd in
     try
       let store = find_field "filename" cd in
-      match ((Ocsigen_config.get_uploaddir ())) with
+      match ci.uploaddir with
         | Some dname ->
             let now = Printf.sprintf "%f-%d"
               (Unix.gettimeofday ()) (counter ()) in
@@ -155,7 +199,7 @@ and find_post_params_multipart_form_data body_gen ctparams filenames =
             in
             Ocsigen_messages.debug2 ("Upload file opened: " ^ fname);
             filenames := fname::!filenames;
-            A_File (p_name, fname, store, fd)
+            A_File (p_name, fname, store, fd, content_type)
         | None -> raise Ocsigen_upload_forbidden
     with Not_found -> No_File (p_name, Buffer.create 1024)
   in
@@ -164,7 +208,7 @@ and find_post_params_multipart_form_data body_gen ctparams filenames =
       | No_File (p_name, to_buf) ->
           Buffer.add_string to_buf s;
           return ()
-      | A_File (_,_,_,wh) ->
+      | A_File (_,_,_,wh,_) ->
           let len = String.length s in
           let r = Unix.write wh s 0 len in
           if r < len then
@@ -178,18 +222,20 @@ and find_post_params_multipart_form_data body_gen ctparams filenames =
         return
           (params := !params @ [(p_name, Buffer.contents to_buf)])
           (*  la fin ? *)
-    | A_File (p_name,fname,oname,wh) ->
+    | A_File (p_name,fname,oname,wh, content_type) ->
         (* Ocsigen_messages.debug "closing file"; *)
         files :=
           !files@[(p_name, {tmp_filename=fname;
                             filesize=size;
                             raw_original_filename=oname;
-                            original_basename=(Ocsigen_lib.basename oname)})];
+                            original_basename=(Ocsigen_lib.basename oname);
+                            file_content_type = content_type;
+                           })];
         Unix.close wh;
         return ()
   in
   Multipart.scan_multipart_body_from_stream
-    body bound create add stop >>= fun () ->
+    body bound create add stop ci.maxuploadfilesize >>= fun () ->
     (*VVV Does scan_multipart_body_from_stream read until the end or
       only what it needs?  If we do not consume here, the following
       request will be read only when this one is finished ...  *)
@@ -199,7 +245,8 @@ and find_post_params_multipart_form_data body_gen ctparams filenames =
 
 
 (* reading the request *)
-let get_request_infos meth clientproto url http_frame filenames sockaddr port receiver =
+let get_request_infos
+    meth clientproto url http_frame filenames sockaddr port receiver =
 
   Lwt.catch
     (fun () ->
@@ -240,7 +287,7 @@ let get_request_infos meth clientproto url http_frame filenames sockaddr port re
 
        let cookies =
          lazy (match (Lazy.force cookies_string) with
-         | None -> Ocsigen_http_frame.Cookievalues.empty
+         | None -> Ocsigen_lib.String_Table.empty
          | Some s -> parse_cookies s)
        in
 
@@ -270,12 +317,20 @@ let get_request_infos meth clientproto url http_frame filenames sockaddr port re
 
        let accept_language = lazy (get_accept_language http_frame) in
 
-       let post_params = lazy
-         (if meth = Http_header.GET || meth = Http_header.HEAD then
-            return ([],[])
-          else
-            find_post_params http_frame ct filenames
-         )
+       let post_params =
+         let r = ref None in
+           (fun ci ->
+              match !r with
+                | None ->
+                    (if meth = Http_header.GET || meth = Http_header.HEAD then
+                       return ([],[])
+                     else
+                       find_post_params http_frame ct filenames ci
+                    ) >>= fun res ->
+                    r := Some res;
+                    return res
+                | Some r -> return r
+           )
        in
 
        let ipstring = Unix.string_of_inet_addr client_inet_addr in
@@ -285,7 +340,7 @@ let get_request_infos meth clientproto url http_frame filenames sockaddr port re
          {ri_url_string = url;
           ri_url = parsed_url;
           ri_method = meth;
-          ri_protocol = http_frame.Ocsigen_http_frame.header.Ocsigen_http_frame.Http_header.proto;
+          ri_protocol = http_frame.Ocsigen_http_frame.frame_header.Ocsigen_http_frame.Http_header.proto;
           ri_ssl = Lwt_ssl.is_ssl (Ocsigen_http_com.connection_fd receiver);
           ri_full_path_string = path_string;
           ri_full_path = path;
@@ -298,10 +353,8 @@ let get_request_infos meth clientproto url http_frame filenames sockaddr port re
           ri_port_from_host_field = headerport;
           ri_get_params = get_params;
           ri_initial_get_params = get_params;
-          ri_post_params = lazy (force post_params >>= fun (a, b) ->
-                                 return a);
-          ri_files = lazy (force post_params >>= fun (a, b) ->
-                           return b);
+          ri_post_params =(fun ci -> post_params ci >>= fun (a, b) -> return a);
+          ri_files = (fun ci -> post_params ci >>= fun (a, b) -> return b);
           ri_remote_inet_addr = client_inet_addr;
           ri_remote_ip = ipstring;
           ri_remote_ip_parsed = lazy (fst (Ocsigen_lib.parse_ip ipstring));
@@ -323,9 +376,10 @@ let get_request_infos meth clientproto url http_frame filenames sockaddr port re
           ri_accept_encoding = accept_encoding;
           ri_accept_language = accept_language;
           ri_http_frame = http_frame;
-          ri_extension_info = [];
+          ri_request_cache = Polytables.create ();
           ri_client = Ocsigen_extensions.client_of_connection receiver;
           ri_range = lazy (Ocsigen_headers.get_range http_frame);
+          ri_nb_tries = 0;
         }
     )
     (fun e ->
@@ -436,7 +490,10 @@ let handle_result_frame ri res send =
         Ocsigen_messages.debug2 "-> Sending 304 Not modified ";
         Ocsigen_stream.finalize (fst res.res_stream) >>= fun () ->
         send { (Ocsigen_http_frame.empty_result ()) with
-                 res_code = 304  (* Not modified *)}
+                 res_code = 304  (* Not modified *);
+                 res_lastmodified = res.res_lastmodified;
+                 res_etag = res.res_etag;
+             }
 
     | `Precondition_failed ->
         Ocsigen_messages.debug2 "-> Sending 412 Precondition Failed \
@@ -456,7 +513,8 @@ let service receiver sender_slot request meth url port sockaddr =
      because the previous one may not be sent *)
 
   let head = meth = Http_header.HEAD in
-  let clientproto = Http_header.get_proto request.Ocsigen_http_frame.header in
+  let clientproto =
+    Http_header.get_proto request.Ocsigen_http_frame.frame_header in
 
   let handle_service_errors e =
     (* Exceptions during page generation *)
@@ -495,6 +553,9 @@ let service receiver sender_slot request meth url port sockaddr =
     | Neturl.Malformed_URL ->
         Ocsigen_messages.debug2 "-> Sending 400 (Malformed URL)";
         send_error 400
+    | Ocsigen_lib.Ocsigen_Request_too_long ->
+        Ocsigen_messages.debug2 "-> Sending 413 (Entity too large)";
+        send_error 413
     | e ->
         Ocsigen_messages.warning
           ("Exn during page generation: " ^ string_of_exn e ^" (sending 500)");
@@ -509,7 +570,7 @@ let service receiver sender_slot request meth url port sockaddr =
          the server writing the response.
        We need to do this once the request has been handled before sending
        any reply to the client. *)
-    match request.Ocsigen_http_frame.content with
+    match request.Ocsigen_http_frame.frame_content with
       | Some f ->
           ignore
             (Lwt.catch
@@ -585,14 +646,15 @@ let service receiver sender_slot request meth url port sockaddr =
                 ri.ri_user_agent
                 ri.ri_url_string);
 
-           let send_aux = send sender_slot ~clientproto ~head
-             ~sender:Ocsigen_http_com.default_sender in
+           let send_aux = 
+             send sender_slot ~clientproto ~head
+               ~sender:Ocsigen_http_com.default_sender
+           in
 
            (* Generation of pages is delegated to extensions: *)
            Lwt.try_bind
-             (fun () ->
-                Ocsigen_extensions.do_for_site_matching
-                  ri.ri_host ri.ri_server_port ri)
+             (fun () -> Ocsigen_extensions.serve_request
+                ~awake_next_request:true ri)
              (fun res ->
                 finish_request ();
                 handle_result_frame ri res send_aux
@@ -692,6 +754,21 @@ let linger in_ch receiver =
 
 let try_bind' f g h = Lwt.try_bind f h g
 
+let add_to_receivers_waiting_for_pipeline, 
+  remove_from_receivers_waiting_for_pipeline,
+  iter_receivers_waiting_for_pipeline =
+  let l = Ocsigen_lib.Clist.create () in
+  ((fun r -> 
+      let node = Ocsigen_lib.Clist.make r in
+      Ocsigen_lib.Clist.insert l node;
+      node),
+   Ocsigen_lib.Clist.remove,
+   (fun f -> 
+      Ocsigen_lib.Clist.fold_left
+        (fun t v -> t >>= fun () -> f v)
+        (Lwt.return ()) 
+        l))
+
 let handle_connection port in_ch sockaddr =
   let receiver = Ocsigen_http_com.create_receiver
     (Ocsigen_config.get_client_timeout ()) Query in_ch
@@ -756,7 +833,7 @@ let handle_connection port in_ch sockaddr =
     end
   in
 
-  let rec handle_request () =
+  let rec handle_request ?receiver_pos () =
     try_bind'
       (fun () ->
          Ocsigen_messages.debug2 "** Receiving HTTP message";
@@ -771,11 +848,21 @@ let handle_connection port in_ch sockaddr =
             Lwt.return ())
          >>= fun () ->
          Ocsigen_http_com.get_http_frame receiver)
-      handle_read_errors
+      (fun exn ->
+         (* We remove the receiver from the set of requests
+            waiting for pipeline *)
+         (match receiver_pos with
+           | Some pos -> remove_from_receivers_waiting_for_pipeline pos
+           | None -> ());
+         handle_read_errors exn)
       (fun request ->
+         (* As above *)
+         (match receiver_pos with
+           | Some pos -> remove_from_receivers_waiting_for_pipeline pos
+           | None -> ());
          let meth, url =
            match 
-             Http_header.get_firstline request.Ocsigen_http_frame.header 
+             Http_header.get_firstline request.Ocsigen_http_frame.frame_header
            with
              | Http_header.Query a -> a
              | _                   -> assert false
@@ -787,8 +874,15 @@ let handle_connection port in_ch sockaddr =
 (*XXX Why do we need the port but not the host name? *)
                 service receiver slot request meth url port sockaddr)
              handle_write_errors);
-         if get_keepalive request.Ocsigen_http_frame.header then
-           handle_request ()
+         if not !shutdown &&
+           get_keepalive request.Ocsigen_http_frame.frame_header
+         then
+           (* We put the receiver in the set of receiver waiting for 
+              pipeline in order to be able to shutdown the connections
+              if the server is shutting down.
+           *)
+           handle_request 
+             ~receiver_pos:(add_to_receivers_waiting_for_pipeline receiver) ()
          else (* No keep-alive => no pipeline *)
             (* We wait for the query to be entirely read and for
                the reply to be sent *)
@@ -801,52 +895,79 @@ let handle_connection port in_ch sockaddr =
   handle_request ()
 
 let rec wait_connection use_ssl port socket =
+  let handle_exn e =
+    Lwt_unix.yield () >>= fun () -> match e with
+    | Socket_closed ->
+        Ocsigen_messages.debug2 "Socket closed";
+        Lwt.return ()
+    | Unix.Unix_error ((Unix.EMFILE | Unix.ENFILE), _, _) ->
+        (* this should not happen, report it *)
+        Ocsigen_messages.errlog
+          "Max number of file descriptors reached unexpectedly, please check...";
+        wait_connection use_ssl port socket
+    | e ->
+        Ocsigen_messages.debug
+          (fun () -> Format.sprintf "Accept failed: %s" (string_of_exn e));
+        wait_connection use_ssl port socket
+  in
   try_bind'
-    (fun () -> Lwt_unix.accept socket)
-    (fun e ->
+    (fun () -> 
+       (* if too much connections,
+          we wait for a signal before accepting again *)
+       let max = get_max_number_of_connections () in
+       (if get_number_of_connected () < max
+        then Lwt.return ()
+        else begin
+          ignore
+            (Ocsigen_messages.warning
+               (Format.sprintf "Max simultaneous connections (%d) reached."
+                  (get_max_number_of_connections ())));
+          wait_fewer_connected max
+        end) >>= fun () ->
+       (* We do several accept(), as explained in 
+         "Accept()able strategies ..." by Tim Brecht & al. *)
+       Lwt_unix.accept_n socket 50)
+    handle_exn
+    (fun (l, e) ->
+       let number_of_accepts = List.length l in
        Ocsigen_messages.debug
-        (fun () -> Format.sprintf "Accept failed: %s" (string_of_exn e));
-       wait_connection use_ssl port socket)
-    (fun (s, sockaddr) ->
-       Ocsigen_messages.debug2
-        "\n__________________NEW CONNECTION__________________________";
-       incr_connected ();
-       let relaunch_at_once =
-         get_number_of_connected () < get_max_number_of_connections () in
-       if relaunch_at_once then
-         ignore (wait_connection use_ssl port socket)
-       else
-         ignore
-           (Ocsigen_messages.warning
-              (Format.sprintf "Max simultaneous connections (%d) reached."
-                 (get_max_number_of_connections ())));
-       Lwt.catch
-         (fun () ->
-            Lwt_unix.set_close_on_exec s;
-            disable_nagle (Lwt_unix.unix_file_descr s);
-            begin if use_ssl then
-              Lwt_ssl.ssl_accept s !sslctx
-            else
-              Lwt.return (Lwt_ssl.plain s)
-            end >>= fun in_ch ->
-            handle_connection port in_ch sockaddr)
-         (fun e ->
-            Ocsigen_messages.unexpected_exception e
-              "Server.wait_connection (handle connection)";
-           return ()) >>= fun () ->
-       Ocsigen_messages.debug2 "** CLOSE";
-       begin try
-         Lwt_unix.close s
-       with Unix.Unix_error _ as e ->
-         Ocsigen_messages.unexpected_exception e "Server.wait_connection (close)"
-       end;
-       decr_connected ();
-       if not relaunch_at_once then
+         (fun () -> "received "^string_of_int number_of_accepts^" accepts" );
+       incr_connected number_of_accepts;
+       ignore (wait_connection use_ssl port socket);
+
+       let handle_one (s, sockaddr) =
+         Ocsigen_messages.debug2
+           "\n__________________NEW CONNECTION__________________________";
+         Lwt.catch
+           (fun () ->
+              Lwt_unix.set_close_on_exec s;
+              disable_nagle (Lwt_unix.unix_file_descr s);
+              begin if use_ssl then
+                Lwt_ssl.ssl_accept s !sslctx
+              else
+                Lwt.return (Lwt_ssl.plain s)
+              end >>= fun in_ch ->
+              handle_connection port in_ch sockaddr)
+           (fun e ->
+              Ocsigen_messages.unexpected_exception e
+                "Server.wait_connection (handle connection)";
+              return ())
+         >>= fun () ->
+         Ocsigen_messages.debug2 "** CLOSE";
          begin
-           debug2 "Ok releasing one connection";
-           ignore (wait_connection use_ssl port socket)
+           try
+             Lwt_unix.close s
+           with Unix.Unix_error _ as e ->
+             Ocsigen_messages.unexpected_exception
+               e "Server.wait_connection (close)"
          end;
-       Lwt.return ())
+         decr_connected ()
+       in
+
+       Lwt_util.iter handle_one l >>= fun () ->
+       match e with
+         | Some e -> handle_exn e
+         | None -> Lwt.return ())
 
 
 
@@ -855,32 +976,11 @@ let stop m n =
 
 (** Thread waiting for events on a the listening port *)
 let listen use_ssl (addr, port) wait_end_init =
-  let listening_socket =
+  let listening_sockets =
     try
-      let socket =
-        try
-          let socket = Lwt_unix.socket Unix.PF_INET6 Unix.SOCK_STREAM 0 in
-          Lwt_unix.set_close_on_exec socket;
-          Lwt_unix.setsockopt socket Unix.SO_REUSEADDR true;
-          Lwt_unix.bind socket (local_addr6 addr port);
-          socket
-        with e ->
-(*VVV CATCH only the IPv6 exception.
-Is it:
-| ENOPROTOOPT  (*  Protocol not available  *) ?
-| EPROTONOSUPPORT  (*  Protocol not supported  *)???
-| ...
-*)
-          Ocsigen_messages.warning
-            ("Exception while creating IPv6 socket: "^Ocsigen_lib.string_of_exn e);
-          let socket = Lwt_unix.socket Unix.PF_INET Unix.SOCK_STREAM 0 in
-          Lwt_unix.set_close_on_exec socket;
-          Lwt_unix.setsockopt socket Unix.SO_REUSEADDR true;
-          Lwt_unix.bind socket (local_addr addr port);
-          socket
-      in
-      Lwt_unix.listen socket 1024;
-      socket
+      let sockets = make_sockets addr port in
+      List.iter (fun x -> Lwt_unix.listen x 1024) sockets;
+      sockets
     with
     | Unix.Unix_error (Unix.EACCES, _, _) ->
         stop
@@ -891,8 +991,10 @@ Is it:
     | exn ->
         stop ("Fatal - Uncaught exception: " ^ string_of_exn exn) 100
   in
-  wait_end_init >>= fun () ->
-  wait_connection use_ssl port listening_socket
+  List.iter (fun x ->
+               ignore (wait_end_init >>= fun () ->
+                       wait_connection use_ssl port x)) listening_sockets;
+  listening_sockets
 
 (* fatal errors messages *)
 let errmsg = function
@@ -929,31 +1031,82 @@ let errmsg = function
 
 
 
-(* reloading the cmo *)
-let reload () =
+(* loading new configuration *)
+let reload_conf s =
+  try
+    Ocsigen_extensions.start_initialisation ();
+    
+    parse_server true s;
+    
+    Ocsigen_extensions.end_initialisation ();
+  with e ->
+    Ocsigen_extensions.end_initialisation ();
+    errlog (fst (errmsg e))
+
+(* reloading the config file *)
+let reload ?file () =
 
   (* That function cannot be interrupted??? *)
   Ocsigen_messages.warning "Reloading config file" ;
 
   (try
-    match parse_config () with
+    match parse_config ?file () with
     | [] -> ()
-    | s::_ ->
-        begin
-          Ocsigen_extensions.start_initialisation ();
-
-          parse_server true s;
-
-          Ocsigen_extensions.end_initialisation ();
-
-        end
-  with e ->
-    Ocsigen_extensions.end_initialisation ();
-    errlog (fst (errmsg e)));
+    | s::_ -> reload_conf s
+  with e -> errlog (fst (errmsg e)));
 
   Ocsigen_messages.warning "Config file reloaded"
 
 
+let shutdown_server s l =
+  try 
+    let timeout = match l with
+      | [] -> Ocsigen_config.get_shutdown_timeout ()
+      | ["notimeout"] -> None
+      | [t] -> 
+          Some (float_of_string t)
+      | _ -> failwith "syntax error in command"
+    in
+    Ocsigen_messages.warning "Shutting down";
+    List.iter 
+      (fun s -> Lwt_unix.abort s Socket_closed) !sockets;
+    List.iter 
+      (fun s -> Lwt_unix.abort s Socket_closed) !sslsockets;
+    sockets := [];
+    sslsockets := [];
+    shutdown := true;
+    if Ocsigen_extensions.get_number_of_connected () <= 0
+    then exit 0;
+    (match timeout with
+       | Some t -> ignore (Lwt_unix.sleep t >>= fun () -> exit 0)
+       | None -> ());
+    ignore
+      (iter_receivers_waiting_for_pipeline
+         (fun receiver ->
+            Ocsigen_http_com.wait_all_senders receiver >>= fun () ->
+              Ocsigen_http_com.abort receiver;
+              Lwt.return ()));
+  with Failure e ->
+    Ocsigen_messages.warning ("Wrong command: " ^ s ^ " (" ^ e ^ ")")
+
+
+let _ =
+  let f s = function
+    | ["reopen_logs"] ->
+        Ocsigen_messages.open_files ();
+        Ocsigen_messages.warning "Log files reopened"
+    | ["reload"] -> reload ()
+    | ["reload"; file] -> reload ~file ()
+    | "shutdown"::l -> shutdown_server s l
+    | ["gc"] ->
+        Gc.compact ();
+        Ocsigen_messages.warning "Heap compaction requested by user"
+    | ["clearcache"] -> Ocsigen_cache.clear_all_caches ()
+    | _ -> raise Ocsigen_extensions.Unknown_command
+  in
+  Ocsigen_extensions.register_command_function f
+
+
 
 let start_server () = try
 
@@ -996,15 +1149,17 @@ let start_server () = try
     Lwt_unix.run
       (let wait_end_init, wait_end_init_awakener = wait () in
       (* Listening on all ports: *)
-      List.iter
-        (fun i ->
-          ignore (listen false i wait_end_init)) ports;
-      List.iter
-        (fun i ->
-          ignore (listen true i wait_end_init)) sslports;
+      sockets := List.fold_left (fun a i -> (listen false i wait_end_init)@a) [] ports;
+      sslsockets := List.fold_left (fun a i -> (listen true i wait_end_init)@a) [] sslports;
 
-      Ocsigen_config.set_ports ports;
-      Ocsigen_config.set_sslports sslports;
+      begin match ports with
+        | (_, p)::_ -> Ocsigen_config.set_default_port p
+        | _ -> ()
+      end;
+      begin match sslports with
+        | (_, p)::_ -> Ocsigen_config.set_default_sslport p
+        | _ -> ()
+      end;
 
       let current_uid = Unix.getuid () in
 
@@ -1076,7 +1231,7 @@ let start_server () = try
       Dynlink_wrapper.prohibit ["Ocsigen_extensions.R"];
       (* As libraries are reloaded each time the config file is read,
          we do not allow to register extensions in libraries *)
-      (* seems it does not work :-( *)
+      (* seems it does not work :-/ *)
 
 
       (* Closing stderr, stdout stdin if silent *)
@@ -1111,25 +1266,29 @@ let start_server () = try
                 [Unix.O_RDWR; Unix.O_NONBLOCK; Unix.O_APPEND] 0o660)) in
 
       let rec f () =
-        Lwt_chan.input_line pipe >>=
-          (fun s ->
-             begin match s with
-               | "reopen_logs" ->
-                   Ocsigen_messages.open_files ();
-                   Ocsigen_messages.warning "Log files reopened"
-               | "reload" -> reload ()
-               | "gc" ->
-                   Gc.compact ();
-                   Ocsigen_messages.warning "Heap compaction requested by user"
-               | _ -> Ocsigen_messages.warning ("Unknown command: " ^ s)
-             end; f ())
+        Lwt_chan.input_line pipe >>= fun s ->
+        Ocsigen_messages.warning ("Command received: "^s);
+        (try
+          let prefix, c =
+            match Ocsigen_lib.split ~multisep:true ' ' s with
+              | [] -> raise Ocsigen_extensions.Unknown_command
+              | a::l ->
+                  try
+                    let aa, ab = Ocsigen_lib.sep ':' a in
+                    (Some aa, (ab::l))
+                  with Not_found -> None, (a::l)
+          in
+          Ocsigen_extensions.get_command_function () ?prefix s c
+        with Unknown_command -> 
+          Ocsigen_messages.warning "Unknown command: ");
+        f ()
       in ignore (f ());
 
-      wakeup wait_end_init_awakener ();
+      Lwt.wakeup wait_end_init_awakener ();
 
-      warning "Ocsigen has been launched (initialisations ok)";
+      Ocsigen_messages.warning "Ocsigen has been launched (initialisations ok)";
 
-      fst (wait ())
+      fst (Lwt.wait ())
       )
   in
 
